
''' A module for representing and manipulating polynomials and their roots.

Provides two classes: Polynomial and PolynomialRoot. '''

import flipper

from math import log10 as log

class Polynomial(object):
	''' This represents an integral polynomial in one variable.
	
	It is specified by a list of coefficients. It is capable of determining
	the number of roots it has in a given interval by using a Sturm chain. '''
	def __init__(self, coefficients):
		assert(isinstance(coefficients, (list, tuple)))
		assert(all(isinstance(coefficient, flipper.IntegerType) for coefficient in coefficients))
		
		# It is easier if the zero polynomial has coefficients [0] instead of [].
		# At various points we will want the min of the coefficients etc.
		if not coefficients: coefficients = [0]
		self.coefficients = list(coefficients[:min(i for i in range(1, len(coefficients)+1) if not any(coefficients[i:]))])
		self.height = log(max(max(abs(x) for x in self.coefficients), 1))
		self.degree = len(self.coefficients) - (2 if self.is_zero() else 1)  # The zero polynomial has degree -1.
		self.log_degree = log(max(self.degree, 1))
		self._chain = None  # Stores the Sturm chain associated to this polynomial. See self.sturm_chain().
	
	def __repr__(self):
		return str(self)
	def __str__(self):
		# So it turns out that displaying polynomials correctly is really hard.
		s = ''
		for index, coefficient in enumerate(self):
			if coefficient != 0:
				pos = coefficient > 0
				power = '^%d' % index if index > 1 else ''
				coeff = '' if abs(coefficient) == 1 else '%d*' % abs(coefficient)
				if index == 0:
					s += str(coefficient)
				elif s == '':
					s += '%s%sx%s' % ('' if pos else '-', coeff, power)
				elif s != '':
					s += ' %s %sx%s' % ('+' if pos else '-', coeff, power)
		
		return '0' if s == '' else s
	def __bool__(self):
		return not self.is_zero()
	def __nonzero__(self):  # For Python2.
		return self.__bool__()
	def __eq__(self, other):
		if isinstance(other, Polynomial):
			return self.degree == other.degree and all(a == b for a, b in zip(self, other))
		elif isinstance(other, flipper.IntegerType):
			return self.degree == 0 and self[0] == other
		else:
			return NotImplemented
	def __ne__(self, other):
		return not (self == other)
	def __hash__(self):
		return hash(tuple(self.coefficients))
	def __neg__(self):
		return Polynomial([-x for x in self])
	
	def __iter__(self):
		return iter(self.coefficients)
	def __len__(self):
		return len(self.coefficients)
	
	def __getitem__(self, item):
		return self.coefficients[item]
	
	def is_zero(self):
		''' Return if this is the zero polynomial. '''
		
		return not any(self)
	
	def __add__(self, other):
		if isinstance(other, Polynomial):
			# Remember to pad the coefficients to ensure none are missed.
			return Polynomial([a + b for a, b in zip(self.coefficients + [0] * len(other), other.coefficients + [0] * len(self))])
		elif isinstance(other, flipper.IntegerType):
			return Polynomial([self[0] + other] + self.coefficients[1:])
		else:
			return NotImplemented
	def __sub__(self, other):
		if isinstance(other, Polynomial):
			# Remember to pad the coefficients to ensure none are missed.
			return Polynomial([a - b for a, b in zip(self.coefficients + [0] * len(other), other.coefficients + [0] * len(self))])
		elif isinstance(other, flipper.IntegerType):
			return Polynomial([self[0] - other] + self.coefficients[1:])
		else:
			return NotImplemented
	def __mul__(self, other):
		if isinstance(other, flipper.IntegerType):
			return Polynomial([a * other for a in self])
		else:
			return NotImplemented
	def __rmul__(self, other):
		return self * other
	
	def shift(self, power):
		''' Return this polynomial multiplied by x^power.
		
		Power must be an integer '''
		
		assert(isinstance(power, flipper.IntegerType))
		
		if power >= 0:
			return Polynomial([0] * power + self.coefficients)
		else:
			return Polynomial(self.coefficients[abs(power):])
	
	def __call__(self, other):
		# It is significantly more efficient to compute self(other) as:
		#   a_0 + x*(a_1 + x*( ... + x*(a_n + x * 0) ... ) )
		# using Horner's rule.
		
		total = 0
		for coefficient in reversed(self):
			total = (other * total) + coefficient
		return total
	def __divmod__(self, other):
		''' Return the quotient and remainder (and rescaling) of this polynomial by other.
		
		That is, returns polynomials quotient and remainder such that:
			scale * self == quotient * other + remainder, and
			deg(remainder) < deg(other)
		If self and other are integral, by Gauss' lemma if other | self then scale == 1. '''
		
		if isinstance(other, Polynomial):
			if other.is_zero():
				raise ZeroDivisionError
			
			# We really should do this calculation modulo some large prime
			# to avoid exponential blowup.
			quotient = Polynomial([0])
			remainder = self
			scale = 1
			while remainder.degree >= other.degree:
				# Now if other | self then Gauss' lemma says this condition will always fail.
				if remainder[-1] % other[-1] != 0:
					remainder = remainder * other[-1]
					quotient = quotient * other[-1]
					scale = scale * other[-1]
				rescale = remainder[-1] // other[-1]
				quotient = quotient + (Polynomial([1]).shift(remainder.degree - other.degree) * rescale)
				remainder = remainder - (other.shift(remainder.degree - other.degree) * rescale)
				common_factor = abs(flipper.kernel.gcd(list(quotient) + list(remainder) + [scale]))
				if common_factor != 1:
					scale = scale // common_factor
					quotient = Polynomial([x // common_factor for x in quotient])
					remainder = Polynomial([x // common_factor for x in remainder])
			
			if scale >= 0:
				return quotient, remainder
			else:
				return -quotient, -remainder
		elif isinstance(other, flipper.IntegerType):
			# Just so self % d will work.
			return None, Polynomial([coefficient % other for coefficient in self])
		else:
			return NotImplemented
	def __mod__(self, other):
		_, remainder = divmod(self, other)
		return remainder
	def __floordiv__(self, other):
		quotient, remainder = divmod(self, other)
		if not remainder.is_zero():  # Division isn't perfect.
			raise ValueError('Polynomials do not divide.')
		return quotient
	def divides(self, other):
		''' Return if self divides other. '''
		
		_, remainder = divmod(other, self)
		return remainder.is_zero()
	
	def gcd(self, other):
		''' Return the GCD(self, other). '''
		
		# Note that doing flipper.kernel.gcd([self, other])
		# may not give the correct answer, it may have the
		# wrong scaling due to how divmod works.
		
		f = flipper.kernel.gcd([self, other]).rescale()
		return f * abs(flipper.kernel.gcd(list(self // f) + list(other // f)))
	
	def rescale(self):
		''' Return a possibly simpler polynomial with the same roots. '''
		
		c = max(abs(flipper.kernel.gcd(self)), 1)  # Avoid a possible division by 0.
		return Polynomial([coefficient // c for coefficient in self])
	
	def square_free(self):
		''' Return the polynomial with the same roots but each with multiplicity one. '''
		
		return self // self.gcd(self.derivative())
	
	def signs_at_interval_endpoints(self, interval):
		''' Return the signs of this polynomial at the endpoints of the given interval. '''
		
		assert(isinstance(interval, flipper.kernel.Interval))
		
		lower, upper, precision = interval.lower, interval.upper, interval.precision
		lower_sign = sum(coefficient * lower**index * 10**(precision*(self.degree - index)) for index, coefficient in enumerate(self))
		upper_sign = sum(coefficient * upper**index * 10**(precision*(self.degree - index)) for index, coefficient in enumerate(self))
		
		return (-1 if lower_sign < 0 else 0 if lower_sign == 0 else +1), (-1 if upper_sign < 0 else 0 if upper_sign == 0 else +1)
	
	def sturm_chain(self):
		''' Return the Sturm chain associated to this polynomial.
		
		This is a finite list of polynomials p_0, p_1, ..., p_m of decreasing degree
		with these following properties:
			p_0 = p is square free,
			if p(x) = 0 then sign(p_1(x)) = sign(p'(x)),
			if p_i(x) = 0 then sign(p_{i-1}(x)) = -sign(p_{i+1}(x)), and
			p_m has constant sign.
		We use Sturm's method for constructing such a chain based off of the Euclidean algorithm, see:
			http://en.wikipedia.org/wiki/Sturm's_theorem '''
		
		if self._chain is None:
			square_free = self.square_free()
			self._chain = [square_free, square_free.derivative()]
			while self._chain[-1]:
				self._chain.append(-(self._chain[-2] % self._chain[-1]).rescale())
		
		return self._chain
	
	def num_roots(self, interval):
		''' Return the number of primitive roots of self in (a, b] where [a, b] is the given interval.
		
		For a description of this variant of the algorithm and an explanation why this works see:
			http://en.wikipedia.org/wiki/Sturm's_theorem '''
		
		chain = self.sturm_chain()
		lower_signs, upper_signs = zip(*[f.signs_at_interval_endpoints(interval) for f in chain])
		lower_non_zero_signs = [x for x in lower_signs if x != 0]
		upper_non_zero_signs = [x for x in upper_signs if x != 0]
		lower_sign_changes = sum(1 for x, y in zip(lower_non_zero_signs, lower_non_zero_signs[1:]) if x * y < 0)
		upper_sign_changes = sum(1 for x, y in zip(upper_non_zero_signs, upper_non_zero_signs[1:]) if x * y < 0)
		return lower_sign_changes - upper_sign_changes
	
	def real_roots(self):
		''' Return a list of (real) PolynomialRoots, one for each root of self.
		
		Repeated roots are not returned multiple times. If you are planning on using
		these roots you should really use self.square_free().real_roots() to ensure
		the Newton-Raphson convergence of these is effective. '''
		
		k = int(self.height + self.log_degree) + 2
		# Roots of self are guaranteed to be separated by at least 10^-k and lie in:
		interval = flipper.kernel.Interval(-10**k, 10**k, 0)
		
		results = []
		to_check = [interval]
		for _ in range(2*k):  # After this many steps all roots must have separated.
			new_to_check = []
			for interval in to_check:
				num_roots = self.num_roots(interval)
				if num_roots > 1:
					new_to_check.extend(interval.subdivide())
				elif num_roots == 1:
					results.append(PolynomialRoot(self, interval))
				else:  # num_roots == 0.
					pass  # So discard this interval.
			to_check = new_to_check
		
		return results
	
	def is_monic(self):
		''' Return if this polynomial is monic or not. '''
		
		return abs(self[-1]) == 1
	
	def derivative(self):
		''' Return the derivative of this polynomial. '''
		
		return Polynomial([a * index for index, a in enumerate(self)][1:])
	
	def companion_matrix(self):
		''' Return the companion matrix of this polynomial.
		
		This is the matrix:
		(  0 |           )
		(----+ -V[:-1]^T )
		( Id |           )
		where V is the vector of coefficients of this polynomial.
		
		Assumes (and checks) that this polynomial is monic. '''
		
		if not self.is_monic():
			raise flipper.AssumptionError('Cannot construct companion matrix for non monic polynomial.')
		
		scale = -1 if self[-1] == 1 else 1
		return flipper.kernel.Matrix([[(scale * self[i]) if j == self.degree-1 else 1 if j == i-1 else 0 for j in range(self.degree)] for i in range(self.degree)])

class PolynomialRoot(object):
	''' This represents a single root of a polynomial.
	
	It is specified by a Polynomial and an Interval.
	
	The interval must contain exactly one root of the polynomial. If
	not an ApproximationError will be raised. These methods are significantly
	faster when the polynomial is square free. '''
	def __init__(self, polynomial, interval):
		assert(isinstance(polynomial, flipper.kernel.Polynomial))
		assert(isinstance(interval, flipper.kernel.Interval))
		
		self.polynomial = polynomial
		self.interval = interval
		self.degree = self.polynomial.degree
		self.log_degree = self.polynomial.log_degree
		self.height = self.polynomial.height + 2 * self.log_degree
		
		# Check that self.polynomial has exactly one root in self.interval.
		if self.polynomial.num_roots(self.interval) != 1:
			raise flipper.ApproximationError('Interval contains %d roots, not one.' % self.polynomial.num_roots(self.interval))
	
	@classmethod
	def from_tuple(cls, coefficients, string):
		''' A short way of constructing PolynomialRoots from a list of coefficients and a string. '''
		
		return cls(Polynomial(coefficients), flipper.kernel.Interval.from_string(string))
	
	def __repr__(self):
		return str(self)
	def __str__(self):
		return 'Root of %s (~%s)' % (self.polynomial, self.approximate_string(6))
	def approximate_string(self, accuracy=None):
		''' Return a string approximating this NumberFieldElement. '''
		
		return self.algebraic_approximation(accuracy).approximate_string(accuracy)
	def __float__(self):
		return float(self.algebraic_approximation())
	def __hash__(self):
		# Just hashing self.polynomial works well as the map:
		#   algebraic number |--> min polynomial
		# is finite-to-one.
		return hash(self.polynomial)
	
	def compare(self, other, operator):
		''' Return the result of operator on sufficiently good algebraic approximations of self and other. '''
		
		if isinstance(other, flipper.IntegerType):
			accuracy_needed = int(self.height + self.log_degree + flipper.kernel.height_int(other) + 0) + 5
			return operator(self.algebraic_approximation(accuracy_needed), other)
		else:
			try:
				accuracy_needed = int(self.height + self.log_degree + other.height + other.log_degree) + 5
				return operator(self.algebraic_approximation(accuracy_needed), other.algebraic_approximation(accuracy_needed))
			except AttributeError:
				return NotImplemented
	
	def __lt__(self, other):
		return self.compare(other, lambda x, y: x < y)
	def __eq__(self, other):
		return self.compare(other, lambda x, y: x == y)
	def __gt__(self, other):
		return self.compare(other, lambda x, y: x > y)
	def __le__(self, other):
		return not (self > other)
	def __ne__(self, other):
		return not (self == other)
	def __ge__(self, other):
		return not (self < other)
	
	def subdivide_iterate(self):
		''' Return a subinterval of self.interval which contains a root of self.polynomial.
		
		We construct this new interval by looking for the interval from self.interval.subdivide()
		which contains a root of self.polynomial. As self.interval contains a unique root
		of self.polynomial the subinterval is also unique. '''
		
		roots = [I for I in self.interval.subdivide() if self.polynomial.num_roots(I) > 0]
		assert(len(roots) == 1)  # What if a root occurs at the end of an interval? It would have to be rational.
		return roots[0]
	
	def newton_raphson_iterate(self):
		''' Return a subinterval of self.interval which contains a root of self.polynomial.
		
		We construct this new interval by applying one step of the (interval) Newton-Raphson
		algorithm to self.interval. This may throw a ZeroDivisionError or ValueError if
		self.interval was not small enough to begin with.
		
		For a description of this variant of the algorithm and an explanation why this works see:
			http://www.rz.uni-karlsruhe.de/~iam/html/language/cxsc/node12.html '''
		
		I = self.interval.midpoint(10)  # Start by building a really tiny interval containing the midpoint of this one.
		J = I - self.polynomial(I) / self.polynomial.derivative()(self.interval)  # Apply the interval NR step. This can throw a ZeroDivisionError.
		K = J.change_denominator(2 * J.accuracy)  # Stop the precision from blowing up too much.
		return K.intersect(self.interval)  # This can throw a ValueError if the intersection is empty.
	
	def converge_iterate(self, accuracy):
		''' Replace self.interval with a subinterval of at least the requested accuracy.
		
		Repeatedly applies self.newton_raphson_iterate to obtain a more accurate subinterval. This should
		converge quadratically so few iterations are needed, if this fails then use self.subdivide_iterate
		before trying again. '''
		
		# At the end we will call self.interval.simplify() which may cost us
		# a point of accuracy, so we will have to iteration until we hit at
		# least accuracy + 1.
		while self.interval.accuracy <= accuracy+1:
			old_accuracy = self.interval.accuracy
			try:
				self.interval = self.newton_raphson_iterate()
			except (ZeroDivisionError, ValueError):
				pass
			
			if self.interval.accuracy == old_accuracy:
				# This is guaranteed to give us at lease 1 more d.p. of accuracy.
				self.interval = self.subdivide_iterate()
		
		# We may lose one point of accuracy here.
		self.interval = self.interval.simplify()
	
	def interval_approximation(self, accuracy=0):
		''' Return an interval containing this root, correct to at least the requested accuracy. '''
		
		min_accuracy = 0
		target_accuracy = max(accuracy, min_accuracy)
		
		if self.interval.accuracy < target_accuracy:
			request_accuracy = target_accuracy
			
			self.converge_iterate(request_accuracy)
			assert(self.interval.accuracy >= target_accuracy)
		
		return self.interval
	
	def algebraic_approximation(self, accuracy=None):
		''' Return an AlgebraicApproximation of this root to at least the requested accuracy. '''
		
		if accuracy is None: accuracy = 0
		min_accuracy = self.height + self.log_degree  # This ensures the AlgebraicApproximation is well defined.
		target_accuracy = max(accuracy, min_accuracy)
		
		request_accuracy = target_accuracy
		
		return flipper.kernel.AlgebraicApproximation(self.interval_approximation(request_accuracy), self.log_degree, self.height)
	
	def square_free_representation(self):
		''' Return this PolynomialRoot but with a square free polynomial.
		
		This can be useful as Newton-Raphson works much better (and so
		convergence happens much faster) for square-free polynomials. '''
		
		return PolynomialRoot(self.polynomial.square_free(), self.interval)
	
	def irreducible_representation(self):
		''' Return this PolynomialRoot but with an irreducible polynomial.
		
		You can make this faster by ensuring that the polynomial is square
		free first.'''
		
		Id = flipper.kernel.id_matrix(self.degree)
		precision = int(self.height + 2*self.degree) + 1
		while True:
			k = 10**precision
			I = self.interval_approximation(precision)
			M = Id.join(flipper.kernel.Matrix([[int(k * I**i) for i in range(self.degree)]])).transpose()
			N = M.LLL()  # This is really slow :(.
			f = flipper.kernel.Polynomial(N[0][:-1])
			if self in f.real_roots():
				return PolynomialRoot(f, I)
			else:
				# This should never happen if we chose precision correctly.
				# However Cohen described this choice as `subtle' so let's be
				# careful and repeat the calculation if we got the wrong answer.
				precision = 2 * precision





''' A module for representing basic ways of changing triangulations.

Provides three classes: Isometry, EdgeFlip and LinearTransformation.

Perhaps in the future we will add a Spiral move so that curves can be
shortened in polynomial time. '''

import flipper

class Move(object):
	def __repr__(self):
		return str(self)
	
	def __call__(self, other):
		if isinstance(other, flipper.kernel.Lamination):
			if other.triangulation != self.source_triangulation:
				raise TypeError('Cannot apply Isometry to a lamination not on the source triangulation.')
			
			return self.target_triangulation.lamination(
				self.apply_geometric(other.geometric),
				self.apply_algebraic(other.algebraic),
				remove_peripheral=False)
		else:
			return NotImplemented
	
	def encode(self):
		''' Return the Encoding induced by this isometry. '''
		
		return flipper.kernel.Encoding([self])

class Isometry(Move):
	''' This represents an isometry from one Triangulation to another.
	
	Triangulations can create the isometries between themselves and this
	is the standard way users are expected to create these. '''
	def __init__(self, source_triangulation, target_triangulation, label_map):
		''' This represents an isometry from source_triangulation to target_triangulation.
		
		It is given by a map taking each edge label of source_triangulation to a label of target_triangulation. '''
		
		assert(isinstance(source_triangulation, flipper.kernel.Triangulation))
		assert(isinstance(target_triangulation, flipper.kernel.Triangulation))
		assert(isinstance(label_map, dict))
		
		self.flip_length = 0  # The number of flips needed to realise this move.
		self.source_triangulation = source_triangulation
		self.target_triangulation = target_triangulation
		self.zeta = self.source_triangulation.zeta
		
		self.label_map = dict(label_map)
		
		# If we are missing any labels then use a depth first search to find the missing ones.
		# Hmmm, should always we do this just to check consistency?
		for i in self.source_triangulation.labels:
			if i not in self.label_map:
				raise flipper.AssumptionError('This label_map not defined on edge %d.' % i)
		
		self.index_map = dict((i, flipper.kernel.norm(self.label_map[i])) for i in self.source_triangulation.indices)
		# Store the inverses too while we're at it.
		self.inverse_label_map = dict((self.label_map[i], i) for i in self.source_triangulation.labels)
		self.inverse_index_map = dict((i, flipper.kernel.norm(self.inverse_label_map[i])) for i in self.source_triangulation.indices)
		self.inverse_signs = dict((i, +1 if self.inverse_index_map[i] == self.inverse_label_map[i] else -1) for i in self.source_triangulation.indices)
	
	def __str__(self):
		return 'Isometry ' + str([self.target_triangulation.edge_lookup[self.label_map[i]] for i in self.source_triangulation.indices])
	def __reduce__(self):
		return (self.__class__, (self.source_triangulation, self.target_triangulation, self.label_map))
	def __len__(self):
		return 1  # The number of pieces of this move.
	def package(self):
		''' Return a small amount of data such that self.source_triangulation.encode([data]) == self.encode(). '''
		
		if not all(self.label_map[i] == i for i in self.source_triangulation.indices):  # If self is not the identity isometry.
			return {i: self.label_map[i] for i in self.source_triangulation.labels}
		else:
			return None
	
	def apply_geometric(self, vector):
		return [vector[self.inverse_index_map[i]] for i in range(self.zeta)]
	def apply_algebraic(self, vector):
		return [vector[self.inverse_index_map[i]] * self.inverse_signs[i] for i in range(self.zeta)]
	
	def inverse(self):
		''' Return the inverse of this isometry. '''
		
		# inverse_corner_map = dict((self(corner), corner) for corner in self.corner_map)
		return Isometry(self.target_triangulation, self.source_triangulation, self.inverse_label_map)
	
	def applied_geometric(self, lamination, action):
		''' Return the action and condition matrices describing the PL map
		applied to the geometric coordinates of the given lamination after
		post-multiplying by the action matrix. '''
		
		assert(isinstance(lamination, flipper.kernel.Lamination))
		assert(isinstance(action, flipper.kernel.Matrix))
		
		return flipper.kernel.Matrix([action[self.inverse_index_map[i]] for i in range(self.zeta)]), flipper.kernel.zero_matrix(0)
	
	def pl_action(self, index, action):
		''' Return the action and condition matrices describing the PL map
		applied to the geometric coordinates by the cell of the specified index
		after post-multiplying by the action matrix. '''
		
		assert(isinstance(index, flipper.IntegerType))
		assert(isinstance(action, flipper.kernel.Matrix))
		
		return (flipper.kernel.Matrix([action[self.inverse_index_map[i]] for i in range(self.zeta)]), flipper.kernel.zero_matrix(0))
	
	def extend_bundle(self, triangulation3, tetra_count, upper_triangulation, lower_triangulation, upper_map, lower_map):
		''' Modify triangulation3 to extend the embedding of upper_triangulation via upper_map under this move. '''
		
		maps_to_triangle = lambda X: isinstance(X[0], flipper.kernel.Triangle)
		maps_to_tetrahedron = lambda X: not maps_to_triangle(X)
		
		# These are the new maps onto the upper and lower boundary that we will build.
		new_upper_map = dict()
		new_lower_map = dict()  # We are allowed to leave blanks in new_lower_map.
		
		for triangle in upper_triangulation:
			new_triangle = self.target_triangulation.triangle_lookup[self.label_map[triangle.labels[0]]]
			new_corner = self.target_triangulation.corner_lookup[self.label_map[triangle.corners[0].label]]
			perm = flipper.kernel.permutation.cyclic_permutation(new_corner.side - 0, 3)
			old_target, old_perm = upper_map[triangle]
			
			if maps_to_triangle(upper_map[triangle]):
				new_upper_map[new_triangle] = (old_target, old_perm * perm.inverse())
				# Don't forget to update the lower_map too.
				new_lower_map[old_target] = (new_triangle, perm * old_perm.inverse())
			else:
				new_upper_map[new_triangle] = (old_target, old_perm * perm.inverse().embed(4))
		
		# Remember to rebuild the rest of lower_map, which hasn't changed.
		for triangle in lower_triangulation:
			if triangle not in new_lower_map:
				new_lower_map[triangle] = lower_map[triangle]
		
		return tetra_count, self.target_triangulation, new_upper_map, new_lower_map

class EdgeFlip(Move):
	''' Represents the change to a lamination caused by flipping an edge. '''
	def __init__(self, source_triangulation, target_triangulation, edge_label):
		assert(isinstance(source_triangulation, flipper.kernel.Triangulation))
		assert(isinstance(target_triangulation, flipper.kernel.Triangulation))
		assert(isinstance(edge_label, flipper.IntegerType))
		
		self.flip_length = 1  # The number of flips needed to realise this move.
		self.source_triangulation = source_triangulation
		self.target_triangulation = target_triangulation
		self.edge_label = edge_label
		self.edge_index = flipper.kernel.norm(self.edge_label)
		self.zeta = self.source_triangulation.zeta
		assert(self.source_triangulation.is_flippable(self.edge_index))
		
		self.square = self.source_triangulation.square_about_edge(self.edge_label)
	
	def __str__(self):
		return 'Flip %s%d' % ('' if self.edge_index == self.edge_label else '~', self.edge_index)
	def __reduce__(self):
		return (self.__class__, (self.source_triangulation, self.target_triangulation, self.edge_label))
	def __len__(self):
		return 2  # The number of pieces of this move.
	def package(self):
		''' Return a small amount of data such that self.source_triangulation.encode([data]) == self.encode(). '''
		
		return self.edge_label
	
	def apply_geometric(self, vector):
		a, b, c, d = self.square
		m = max(vector[a.index] + vector[c.index], vector[b.index] + vector[d.index]) - vector[self.edge_index]
		return [vector[i] if i != self.edge_index else m for i in range(self.zeta)]
	def apply_algebraic(self, vector):
		a, b, c, d = self.square
		m = b.sign() * vector[b.index] + c.sign() * vector[c.index]
		return [vector[i] if i != self.edge_index else m for i in range(self.zeta)]
	
	def inverse(self):
		''' Return the inverse of this map. '''
		
		return EdgeFlip(self.target_triangulation, self.source_triangulation, ~self.edge_label)
	
	def applied_geometric(self, lamination, action):
		''' Return the action and condition matrices describing the PL map
		applied to the geometric coordinates of the given lamination after
		post-multiplying by the action matrix. '''
		
		assert(isinstance(lamination, flipper.kernel.Lamination))
		assert(isinstance(action, flipper.kernel.Matrix))
		
		a, b, c, d, e = [edge.index for edge in self.square] + [self.edge_index]
		
		rows = [list(row) for row in action]
		if lamination(a) + lamination(c) >= lamination(b) + lamination(d):
			rows[e] = [rows[a][i] + rows[c][i] - rows[e][i] for i in range(self.zeta)]
			Cs = flipper.kernel.Matrix([[action[a][i] + action[c][i] - action[b][i] - action[d][i] for i in range(self.zeta)]])
		else:
			rows[e] = [rows[b][i] + rows[d][i] - rows[e][i] for i in range(self.zeta)]
			Cs = flipper.kernel.Matrix([[action[b][i] + action[d][i] - action[a][i] - action[c][i] for i in range(self.zeta)]])
		return flipper.kernel.Matrix(rows), Cs
	
	def pl_action(self, index, action):
		''' Return the action and condition matrices describing the PL map
		applied to the geometric coordinates by the cell of the specified index
		after post-multiplying by the action matrix. '''
		
		assert(isinstance(index, flipper.IntegerType))
		assert(isinstance(action, flipper.kernel.Matrix))
		
		a, b, c, d, e = [edge.index for edge in self.square] + [self.edge_index]
		
		rows = [list(row) for row in action]
		if index == 0:
			rows[e] = [rows[a][i] + rows[c][i] - rows[e][i] for i in range(self.zeta)]
			Cs = flipper.kernel.Matrix([[action[a][i] + action[c][i] - action[b][i] - action[d][i] for i in range(self.zeta)]])
		elif index == 1:
			rows[e] = [rows[b][i] + rows[d][i] - rows[e][i] for i in range(self.zeta)]
			Cs = flipper.kernel.Matrix([[action[b][i] + action[d][i] - action[a][i] - action[c][i] for i in range(self.zeta)]])
		else:
			raise IndexError('Index out of range.')
		return flipper.kernel.Matrix(rows), Cs
	
	def extend_bundle(self, triangulation3, tetra_count, upper_triangulation, lower_triangulation, upper_map, lower_map):
		
		''' Modify triangulation3 to extend the embedding of upper_triangulation via upper_map under this move. '''
		
		assert(upper_triangulation == self.source_triangulation)
		
		# We use these two functions to quickly tell what a triangle maps to.
		maps_to_triangle = lambda X: isinstance(X[0], flipper.kernel.Triangle)
		maps_to_tetrahedron = lambda X: not maps_to_triangle(X)
		
		# These are the new maps onto the upper and lower boundary that we will build.
		new_upper_map = dict()
		new_lower_map = dict()
		# We are allowed to leave blanks in new_lower_map.
		# These will be filled in at the end using lower_map.
		new_upper_triangulation = self.target_triangulation
		VEERING_LEFT, VEERING_RIGHT = flipper.kernel.triangulation3.VEERING_LEFT, flipper.kernel.triangulation3.VEERING_RIGHT
		
		# Get the next tetrahedra to add.
		tetrahedron = triangulation3.tetrahedra[tetra_count]
		
		# Setup the next tetrahedron.
		tetrahedron.edge_labels[(0, 1)] = VEERING_RIGHT
		tetrahedron.edge_labels[(1, 2)] = VEERING_LEFT
		tetrahedron.edge_labels[(2, 3)] = VEERING_RIGHT
		tetrahedron.edge_labels[(0, 3)] = VEERING_LEFT
		
		edge_label = self.edge_label  # The edge to flip.
		
		# We'll glue it into the core_triangulation so that it's 1--3 edge lies over edge_label.
		# WARNINNG: This is reliant on knowing how flipper.kernel.Triangulation.flip_edge() relabels things!
		cornerA = upper_triangulation.corner_of_edge(edge_label)
		cornerB = upper_triangulation.corner_of_edge(~edge_label)
		
		# We'll need to swap sides on an inverse edge so our convertions below work.
		if edge_label != self.edge_index: cornerA, cornerB = cornerB, cornerA
		
		(A, side_A), (B, side_B) = (cornerA.triangle, cornerA.side), (cornerB.triangle, cornerB.side)
		if maps_to_tetrahedron(upper_map[A]):
			tetra, perm = upper_map[A]
			tetrahedron.glue(2, tetra, flipper.kernel.permutation.permutation_from_pair(0, perm(side_A), 2, perm(3)))
		else:
			tri, perm = upper_map[A]
			new_lower_map[tri] = (tetrahedron, flipper.kernel.permutation.permutation_from_pair(perm(side_A), 0, 3, 2))
		
		if maps_to_tetrahedron(upper_map[B]):
			tetra, perm = upper_map[B]
			# The permutation needs to: 2 |--> perm(3), 0 |--> perm(side_A), and be odd.
			tetrahedron.glue(0, tetra, flipper.kernel.permutation.permutation_from_pair(2, perm(side_B), 0, perm(3)))
		else:
			tri, perm = upper_map[B]
			new_lower_map[tri] = (tetrahedron, flipper.kernel.permutation.permutation_from_pair(perm(side_B), 2, 3, 0))
		
		# Rebuild the upper_map.
		new_cornerA = new_upper_triangulation.corner_of_edge(edge_label)
		new_cornerB = new_upper_triangulation.corner_of_edge(~edge_label)
		new_A, new_B = new_cornerA.triangle, new_cornerB.triangle
		# Most of the triangles have stayed the same.
		# This relies on knowing how the upper_triangulation.flip_edge() function works.
		old_fixed_triangles = [triangle for triangle in upper_triangulation if triangle != A and triangle != B]
		new_fixed_triangles = [triangle for triangle in new_upper_triangulation if triangle != new_A and triangle != new_B]
		for old_triangle, new_triangle in zip(old_fixed_triangles, new_fixed_triangles):
			new_upper_map[new_triangle] = upper_map[old_triangle]
			if maps_to_triangle(upper_map[old_triangle]):  # Don't forget to update the lower_map too.
				target_triangle, perm = upper_map[old_triangle]
				new_lower_map[target_triangle] = (new_triangle, perm.inverse())
		
		# This relies on knowing how the upper_triangulation.flip_edge() function works.
		perm_A = flipper.kernel.permutation.cyclic_permutation(new_upper_triangulation.corner_of_edge(edge_label).side, 3)
		perm_B = flipper.kernel.permutation.cyclic_permutation(new_upper_triangulation.corner_of_edge(~edge_label).side, 3)
		new_upper_map[new_A] = (tetrahedron, flipper.kernel.Permutation((3, 0, 2, 1)) * perm_A.embed(4).inverse())
		new_upper_map[new_B] = (tetrahedron, flipper.kernel.Permutation((1, 2, 0, 3)) * perm_B.embed(4).inverse())
		
		# Remember to rebuild the rest of lower_map, which hasn't changed.
		for triangle in lower_triangulation:
			if triangle not in new_lower_map:
				new_lower_map[triangle] = lower_map[triangle]
		
		return tetra_count+1, self.target_triangulation, new_upper_map, new_lower_map

class Spiral(Move):
	''' This represents a spiral around a short curve. '''
	def __init__(self, source_triangulation, target_triangulation, edge_label, power):
		''' This represents spiralling around a short curve passing through edge_label.
		
		The number of spirals is determined by power and the compact form of this move
		means that the amount of work to compute the image of a lamination under this move
		is logorithmic in the power.
		
		Because this is a mapping class, source_triangulation and target_triangulation should be equal. '''
		
		assert(isinstance(source_triangulation, flipper.kernel.Triangulation))
		assert(isinstance(target_triangulation, flipper.kernel.Triangulation))
		assert(source_triangulation == target_triangulation)
		
		self.flip_length = abs(power)  # The number of flips needed to realise this move.
		self.source_triangulation = source_triangulation
		self.target_triangulation = target_triangulation
		self.zeta = self.source_triangulation.zeta
		
		self.edge_label = edge_label
		self.edge_index = flipper.kernel.norm(self.edge_label)
		# Find a, b, c & e automatically.
		self.square = self.source_triangulation.square_about_edge(self.edge_label)
		a, b, c, d = self.square
		assert(b == ~d)
		# Assert that b == d.
		self.power = power
	
	def __str__(self):
		return 'Spiral^%d %s' % (self.power, self.edge_label)
	def __reduce__(self):
		return (self.__class__, (self.source_triangulation, self.target_triangulation, self.edge_label, self.power))
	def __len__(self):
		return abs(self.power) + 3  # The number of pieces of this move.
	def package(self):
		''' Return a small amount of data such that self.source_triangulation.encode([data]) == self.encode(). '''
		
		return (self.edge_label, self.power)
	
	def mat(self, config, t):
		k = abs(self.power)
		if k == 0: return flipper.kernel.id_matrix(4)
		
		def F(n):  # Note F(0) == Id.
			return flipper.kernel.Matrix([
				[1,0,  0,  0],
				[0,1,  0,  0],
				[0,0,n+1, -n],
				[0,0,  n,1-n]
				])  # (Un)Stable transitions.
		G = flipper.kernel.Matrix([
			[1,0,0, 0],
			[0,1,0, 0],
			[1,1,0,-1],
			[0,0,1, 0]])  # Temp transitions.
		
		# Take k steps in the graph. Leave unstable after t steps.
		if config == 1:  # Stable.
			M = F(k)
		elif config == 2:  # Transition.
			M = F(k-1) * G
		elif config == 3:  # Unstable.
			if k <= t:  # k steps in unstable
				M = F(k)
			elif k == t+1:  # t steps in unstable, then into transition.
				M = G * F(t)
			elif k == t+2:  # t steps in unstable, into transition, then out of transition.
				M = G * G * F(t)
			else:  # t+2 < k:  # t steps in unstable, into transition, out of transition, then the remaining steps in stable.
				M = F(k - (t + 2)) * G * G * F(t)
		
		return M
	
	def apply_geometric(self, vector):
		# We will begin with an easy case so we can later assume self.power != 0.
		
		ai, bi, ci, di = [edge.index for edge in self.square]
		ei = self.edge_index
		a, b, c, d = [vector[edge.index] for edge in self.square]
		e = vector[self.edge_index]
		
		# Determine the number of strands passing through the annulus.
		x = max(b - e, a + c - b - e, e - b)
		# Use that to determine the configuration we're in.
		# There are three possible stats:
		#  1: Stable
		#  2: Transitioning
		#  3: Unstable
		# We use this slightly unusual ordering to ensure this gives the
		# same preference (a + c >= b + d) as EdgeFlip.
		state = 2 if x == a+c-b-e else 1 if x == b-e else 3
		
		k = abs(self.power)
		
		# Compute action on a, b, c, e.
		# Note that if self.power > 0 then we use the ordering a, c, b, e
		# and otherwise we use a, c, e, d. This allows us to do two calculations
		# with only one matrix.
		# Additionally d == b so we dont need to compute new_d.
		
		# The maximum number of times you can perform the unstable state.
		# WLOG 0 <= t <= k.
		
		if self.power > 0:
			config = state
			t = min(max((2*b - a - c) // (2*(e - b)) + 1, 0), k) if e != b else k
			M = self.mat(config, t)  # t only matters if in the unstable, that is, if config == 3.
			_, _, new_b, new_e = M([a, c, b, e])
		else:
			# Reverse the configuration if we are taking a negative power.
			config = 4 - state
			t = min(max((2*e - a - c) // (2*(b - e)) + 1, 0), k) if e != b else k
			M = self.mat(config, t)  # t only matters if in the unstable, that is, if config == 3.
			_, _, new_e, new_b = M([a, c, e, b])
		
		return [new_b if i == bi else new_e if i == ei else vector[i] for i in range(self.zeta)]
	
	def apply_algebraic(self, vector):
		a, b, c, d = self.square
		e = self.source_triangulation.edge_lookup[self.edge_label]
		new_b = vector[b.index] + b.sign() * c.sign() * vector[c.index] * self.power
		new_e = vector[e.index] - e.sign() * c.sign() * vector[c.index] * self.power
		return [new_b if i == b.index else new_e if i == self.edge_index else vector[i] for i in range(self.zeta)]
	
	def inverse(self):
		''' Return the inverse of this isometry. '''
		
		# inverse_corner_map = dict((self(corner), corner) for corner in self.corner_map)
		return Spiral(self.target_triangulation, self.source_triangulation, self.edge_label, -self.power)
	
	def applied_geometric(self, lamination, action):
		''' Return the action and condition matrices describing the PL map
		applied to the geometric coordinates of the given lamination after
		post-multiplying by the action matrix. '''
		
		assert(isinstance(lamination, flipper.kernel.Lamination))
		assert(isinstance(action, flipper.kernel.Matrix))
		
		a, b, c, d, e = [lamination(edge) for edge in self.square] + [lamination(self.edge_index)]
		x = max(b - e, a + c - b - e, e - b)
		state = 2 if x == a+c-b-e else 1 if x == b-e else 3
		config = state if self.power >= 0 else 4 - state
		k = abs(self.power)
		
		if self.power > 0:
			t = min(max((2*b - a - c) // (2*(e - b)) + 1, 0), k) if e != b else k
		else:
			t = min(max((2*e - a - c) // (2*(b - e)) + 1, 0), k) if e != b else k
		
		index = 0 if config == 1 else 1 if config == 2 else t + 2
		
		return self.pl_action(index, action)
	
	def pl_action(self, index, action):
		''' Return the action and condition matrices describing the PL map
		applied to the geometric coordinates by the cell of the specified index
		after post-multiplying by the action matrix. '''
		
		assert(isinstance(index, flipper.IntegerType))
		assert(isinstance(action, flipper.kernel.Matrix))
		
		if index == 0:
			config, t = 1, 0
		elif index == 1:
			config, t = 2, 0
		else:  # index >= 2.
			config, t = 3, index - 2
		
		state = config if self.power > 0 else 4 - config
		
		ai, bi, ci, di = [edge.index for edge in self.square]
		ei = self.edge_index
		k = abs(self.power)
		
		if self.power >= 0:
			M = self.mat(config, t)  # t only matters if in the unstable, that is, if config == 3.
			new_b_row = [M[2][0]*action[ai][i] + M[2][1]*action[ci][i] + M[2][2]*action[bi][i] + M[2][3]*action[ei][i] for i in range(self.zeta)]
			new_e_row = [M[3][0]*action[ai][i] + M[3][1]*action[ci][i] + M[3][2]*action[bi][i] + M[3][3]*action[ei][i] for i in range(self.zeta)]
		else:
			M = self.mat(config, t)  # t only matters if in the unstable, that is, if config == 3.
			new_e_row = [M[2][0]*action[ai][i] + M[2][1]*action[ci][i] + M[2][2]*action[ei][i] + M[2][3]*action[bi][i] for i in range(self.zeta)]
			new_b_row = [M[3][0]*action[ai][i] + M[3][1]*action[ci][i] + M[3][2]*action[ei][i] + M[3][3]*action[bi][i] for i in range(self.zeta)]
		A = flipper.kernel.Matrix([new_b_row if i == bi else new_e_row if i == ei else action[i] for i in range(self.zeta)])
		
		if config == 1:
			if state == 1:
				Cs = flipper.kernel.Matrix([
					[2*action[bi][i] - action[ai][i] - action[ci][i] for i in range(self.zeta)],
					[action[bi][i] - action[ei][i] for i in range(self.zeta)]
					])
			else:  # state == 3.
				Cs = flipper.kernel.Matrix([
					[2*action[ei][i] - action[ai][i] - action[ci][i] for i in range(self.zeta)],
					[action[ei][i] - action[bi][i] for i in range(self.zeta)]
					])
		elif config == 2:
			Cs = flipper.kernel.Matrix([
				[-2*action[bi][i] + action[ai][i] + action[ci][i] for i in range(self.zeta)],
				[-2*action[ei][i] + action[ai][i] + action[ci][i] for i in range(self.zeta)],
				])
		else:  # config == 3.
			# Construct Cs so t has the correct value.
			if state == 1:
				Cs = flipper.kernel.Matrix([
					[2*action[bi][i] - action[ai][i] - action[ci][i] for i in range(self.zeta)],
					[action[bi][i] - action[ei][i] for i in range(self.zeta)]
					])
				if t != 0:
					Cs = Cs.join(flipper.kernel.Matrix([
						[-2*(t-1)*action[bi][i] - action[ai][i] - action[ci][i] + 2*t*action[ei][i] for i in range(self.zeta)]
						]))
				if t != k:
					Cs = Cs.join(flipper.kernel.Matrix([
						[2*t*action[bi][i] + action[ai][i] + action[ci][i] - 2*(t+1)*action[ei][i] for i in range(self.zeta)]
						]))
			else:  # state == 3.
				Cs = flipper.kernel.Matrix([
					[2*action[ei][i] - action[ai][i] - action[ci][i] for i in range(self.zeta)],
					[action[ei][i] - action[bi][i] for i in range(self.zeta)]
					])
				if t != 0:
					Cs = Cs.join(flipper.kernel.Matrix([
						[-2*(t-1)*action[ei][i] - action[ai][i] - action[ci][i] + 2*t*action[bi][i] for i in range(self.zeta)]
						]))
				if t != k:
					Cs = Cs.join(flipper.kernel.Matrix([
						[2*t*action[ei][i] + action[ai][i] + action[ci][i] - 2*(t+1)*action[bi][i] for i in range(self.zeta)]
						]))
		
		return A, Cs
	
	def extend_bundle(self, triangulation3, tetra_count, upper_triangulation, lower_triangulation, upper_map, lower_map):
		''' Modify triangulation3 to extend the embedding of upper_triangulation via upper_map under this move. '''
		
		# These are the new maps onto the upper and lower boundary that we will build.
		
		if self.power == 0:
			return tetra_count, upper_triangulation, upper_map, lower_map
		
		bi, ei = self.square[1].index, self.edge_index
		k = abs(self.power)
		
		if self.power > 0:
			sequence = ([bi, ei] * k)[-k:]
		else:  # self.power < 0:
			sequence = ([ei, bi] * k)[-k:]
		
		twist = upper_triangulation.encode([{i: i for i in upper_triangulation.indices if i not in [ei, bi]}] + sequence)
		for item in reversed(twist.sequence):
			tetra_count, upper_triangulation, upper_map, lower_map = \
				item.extend_bundle(triangulation3, tetra_count, upper_triangulation, lower_triangulation, upper_map, lower_map)
		
		assert(upper_triangulation == self.target_triangulation)
		
		return tetra_count, self.target_triangulation, upper_map, lower_map

class LinearTransformation(Move):
	''' Represents the change to a lamination caused by a linear map. '''
	def __init__(self, source_triangulation, target_triangulation, geometric, algebraic):
		assert(isinstance(source_triangulation, flipper.kernel.Triangulation))
		assert(isinstance(target_triangulation, flipper.kernel.Triangulation))
		assert(isinstance(geometric, flipper.kernel.Matrix))
		assert(isinstance(algebraic, flipper.kernel.Matrix))
		
		self.flip_length = 0  # The number of flips needed to realise this move.
		self.source_triangulation = source_triangulation
		self.target_triangulation = target_triangulation
		self.geometric = geometric
		self.algebraic = algebraic
	
	def __str__(self):
		return str(self.geometric) + str(self.algebraic)
	def __len__(self):
		return 1  # The number of pieces of this move.
	def package(self):
		''' Return a small amount of data such that self.source_triangulation.encode([data]) == self.encode(). '''
		
		return self
	
	def apply_geometric(self, vector):
		return self.geometric(vector)
	def apply_algebraic(self, vector):
		return self.algebraic(vector)
	
	def inverse(self):
		''' Return the inverse of this map.
		
		Note that these do not exist and so NotImplemented is returned. '''
		
		return NotImplemented
	
	def applied_geometric(self, lamination, action):
		''' Return the action and condition matrices describing the PL map
		applied to the geometric coordinates of the given lamination after
		post-multiplying by the action matrix. '''
		
		assert(isinstance(lamination, flipper.kernel.Lamination))
		assert(isinstance(action, flipper.kernel.Matrix))
		
		return self.geometric * action, flipper.kernel.zero_matrix(0)
	
	def pl_action(self, index, action):
		''' Return the action and condition matrices describing the PL map
		applied to the geometric coordinates by the cell of the specified index
		after post-multiplying by the action matrix. '''
		
		assert(isinstance(index, flipper.IntegerType))
		assert(isinstance(action, flipper.kernel.Matrix))
		
		return (self.geometric * action, flipper.kernel.zero_matrix(0))





from __future__ import print_function
from time import time
import cProfile
import pstats

import flipper

NT_TYPE_PERIODIC = flipper.kernel.encoding.NT_TYPE_PERIODIC
NT_TYPE_REDUCIBLE = flipper.kernel.encoding.NT_TYPE_REDUCIBLE
NT_TYPE_PSEUDO_ANOSOV = flipper.kernel.encoding.NT_TYPE_PSEUDO_ANOSOV

def main(verbose=False):
	if verbose: print('Running equippedtriangulation profile.')
	
	start_time = time()
	# Add more tests here.
	
	tests = [
		('S_1_1', 10, {}),
		('S_1_2', 6, {}),
		('S_0_4', 4, {'letters': ['a', 'b']}),
		]
	
	for surface, depth, options in tests:
		sum(1 for _ in flipper.load(surface).all_words(depth, **options))
	
	return time() - start_time

if __name__ == '__main__':
	# main()
	# print(main(verbose=True))
	#pstats.Stats(cProfile.Profile().run('main()')).strip_dirs().sort_stats('time').print_callers(20)
	#pstats.Stats(cProfile.Profile().run('main()')).strip_dirs().sort_stats('cumtime').print_callees(20)
	pstats.Stats(cProfile.Profile().run('main()')).strip_dirs().sort_stats('time').print_stats(30)





from __future__ import print_function

import flipper
try:
	import snappy
except ImportError:
	snappy = None

def main():
	surface, word = 'S_1_2', 'abC'
	# Get an example mapping class - this one we know is pseudo-Anosov.
	# This process will fail with an AssumptionError if our map is not pseudo-Anosov.
	h = flipper.load(surface).mapping_class(word)
	print('Built the mapping class h := \'%s\' on %s.' % (word, surface))
	# Get the bundle of h.
	bundle = h.bundle()
	print('Built the canonical triangulation of the bundle with monodromy h.')
	print('This triangulation %s veering.' % ('is' if bundle.is_veering() else 'is not'))
	print('It has %d cusp(s) with the following properties:' % bundle.num_cusps)
	for index, (real, fibre, degeneracy) in enumerate(zip(bundle.cusp_types(), bundle.fibre_slopes(), bundle.degeneracy_slopes())):
		print('\tCusp %s (%s): Fibre slope %s, degeneracy slope %s' % (index, 'Real' if real else 'Fake', fibre, degeneracy))
	
	M = snappy.Manifold(bundle.snappy_string())
	print('The manifold M: %s' % M)
	print('has these fake cusps filled along the fibre slopes.')
	print('Hence M is actually the mapping torus M_h.')
	print('Snappy also knows this manifold as:')
	print('\t%s' % M.identify())
	
	N = snappy.Manifold(bundle.snappy_string(filled=False))
	print('The manifold N: %s' % N)
	print('has these cusps left unfilled.')

if __name__ == '__main__':
	import argparse
	parser = argparse.ArgumentParser(description='Build a bundle and verify it with SnapPy')
	parser.add_argument('--show', action='store_true', default=False, help='show the source code of this example and exit')
	args = parser.parse_args()
	
	if args.show:
		print(open(__file__, 'r').read())
	else:
		if snappy is None:
			print('This example requires SnapPy.')
			print('Please install it and try again.')
		else:
			main()





from math import sqrt
from random import random
from colorsys import hls_to_rgb

PHI = (1 + sqrt(5)) / 2

DEFAULT_OBJECT_COLOUR = 'black'
DEFAULT_VERTEX_COLOUR = 'black'
DEFAULT_EDGE_COLOUR = 'black'
DEFAULT_TRIANGLE_COLOUR = 'gray80'
DEFAULT_CURVE_COLOUR = 'grey40'
DEFAULT_TRAIN_TRACK_BLOCK_COLOUR = 'grey40'

ARROW_FRAC = 0.55

def dot(a, b):
	return a[0] * b[0] + a[1] * b[1]

def lines_intersect(s1, e1, s2, e2, float_error, equivalent_edge):
	dx1, dy1 = e1[0] - s1[0], e1[1] - s1[1]
	dx2, dy2 = e2[0] - s2[0], e2[1] - s2[1]
	D = dx2*dy1 - dx1*dy2
	if D == 0: return (-1, False)
	
	xx = s2[0] - s1[0]
	yy = s2[1] - s1[1]
	
	s = float(yy*dx1 - xx*dy1)/D
	t = float(yy*dx2 - xx*dy2)/D
	
	return (t if 0-float_error <= s <= 1+float_error and 0-float_error <= t <= 1+float_error else -1, equivalent_edge and 0+float_error <= s <= 1-float_error and 0+float_error <= t <= 1-float_error)

def intersection(A, d, B, d2):
	# Find the intersection parameters of A + t d and B + t2 d2
	# Want t & t2 such that:
	#   A[0] + t d[0] = B[0] + t2 d2[0]
	#   A[1] + t d[1] = B[1] + t2 d2[1]
	# So:
	#   (d[0] -d2[0]) (t ) = (B[0] - A[0])
	#   (d[1] -d2[1]) (t2)   (B[1] - A[1])
	# The inverse of this matrix is:
	#   (-d2[1] d2[0])
	#   ( -d[1]  d[0]) / det
	# where:
	det = d2[0] * d[1] - d[0] * d2[1]
	# So:
	t  = ((B[0] - A[0]) * -d2[1] + (B[1] - A[1]) * d2[0]) / det
	t2 = ((B[0] - A[0]) *  -d[1] + (B[1] - A[1]) *  d[0]) / det
	
	return t, t2

def interpolate(A, B, C, r, s):
	# Given points A, B, C and parameters r, s
	# Let X := rB + (1-r)A and
	# Y := sB + (1-s)C
	dx, dy = A[0] - B[0], A[1] - B[1]
	dx2, dy2 = C[0] - B[0], C[1] - B[1]
	
	X = (B[0] + r*dx, B[1] + r*dy)
	Y = (B[0] + s*dx2, B[1] + s*dy2)
	
	d1a = intersection(X, (-dy, dx), B, (dx+dx2, dy+dy2))[0]
	d1b = intersection(X, (-dy, dx), A, (-2*A[0] + B[0] + C[0], -2*A[1] + B[1] + C[1]))[0]
	t = min([x for x in [d1a, d1b] if x > 0]) / 2
	
	d2a = intersection(Y, (dy2, -dx2), B, (dx+dx2, dy+dy2))[0]
	d2b = intersection(Y, (dy2, -dx2), C, (-2*C[0] + A[0] + B[0], -2*C[1] + A[1] + B[1]))[0]
	t2 = min([x for x in [d2a, d2b] if x > 0]) / 2
	
	P = (X[0] + t * -dy, X[1] + t * dx)
	Q = (Y[0] + t2 * dy2, Y[1] + t2 * -dx2)
	
	return X, P, Q, Y

class ColourPalette(object):
	def __init__(self):
		self.state = 0
	
	def get_colour(self):
		self.state += 1
		hue = (self.state / PHI) % 1.
		lightness = (50 + random() * 10)/100.
		saturation = (90 + random() * 10)/100.
		r, g, b = hls_to_rgb(hue, lightness, saturation)
		return '#%02x%02x%02x' % (int(r * 255), int(g * 255), int(b * 255))
	
	def reset(self):
		self.state = 0

class DrawableObject(object):
	def __init__(self, canvas, vertices, options):
		self.options = options
		self.canvas = canvas
		self.vertices = vertices
		self.default_colour = DEFAULT_OBJECT_COLOUR
		self.colour = None
		self.drawn = None
	
	# Note that this means that CanvasTriangle will NOT have the same convention as AbstractTriangle,
	# there iterating and index accesses return edges.
	def __getitem__(self, index):
		return self.vertices[index % len(self)]
	
	def __iter__(self):
		return iter(self.vertices)
	
	def __len__(self):
		return len(self.vertices)
	
	def set_current_colour(self, colour=None):
		if colour is None: colour = self.colour
		self.canvas.itemconfig(self.drawn, fill=colour)
	
	def set_colour(self, colour=None):
		if colour is None: colour = self.default_colour
		self.colour = colour
		self.set_current_colour(self.colour)
	
	def centre(self):
		return (sum(x[0] for x in self.vertices) / len(self), sum(x[1] for x in self.vertices) / len(self))
	
	def update(self):
		self.canvas.coords(self.drawn, *[c for v in self for c in v])

class CanvasVertex(DrawableObject):
	def __init__(self, canvas, vertex, options):
		super(CanvasVertex, self).__init__(canvas, [vertex], options)
		self.default_colour = self.colour = DEFAULT_VERTEX_COLOUR
		self.vertex = list(vertex)
		self.drawn = self.canvas.create_oval(
			[p + scale*self.options.dot_size for scale in [-1, 1] for p in self],
			outline=self.default_colour, fill=self.default_colour, tag='oval'
			)
	
	def __repr__(self):
		return str(self.vertex)
	
	def __sub__(self, other):
		return (self[0] - other[0], self[1] - other[1])
	
	# We have to redo these manually.
	def __iter__(self):
		return iter(self.vertex)
	
	def __getitem__(self, key):
		return self.vertex[key]
	
	def __setitem__(self, key, value):
		self.vertex[key] = value
	
	def __contains__(self, point):
		return all(abs(c - v) < self.options.epsilon for c, v in zip(point, self))
	
	def update(self):
		self.canvas.coords(self.drawn, *[p + scale*self.options.dot_size for scale in [-1, 1] for p in self])

class CanvasEdge(DrawableObject):
	def __init__(self, canvas, vertices, options):
		super(CanvasEdge, self).__init__(canvas, vertices, options)
		self.default_colour = self.colour = DEFAULT_EDGE_COLOUR
		m = ((1-ARROW_FRAC)*self.vertices[0][0] + ARROW_FRAC*self.vertices[1][0], (1-ARROW_FRAC)*self.vertices[0][1] + ARROW_FRAC*self.vertices[1][1])
		self.drawn = [
			self.canvas.create_line(
				[c for v in [self.vertices[0], m] for c in v],
				width=self.options.line_size,
				fill=self.default_colour,
				tags=['line', 'line_start']
			),
			self.canvas.create_line(
				[c for v in [self.vertices[0], self.vertices[1]] for c in v],
				width=self.options.line_size,
				fill=self.default_colour,
				tags=['line', 'line_end']
			)
			]
		
		self.equivalent_edge = None
		self.in_triangles = []
		self.index = -1
	
	def __contains__(self, point):
		if any(point in vertex for vertex in self):
			return False
		try:
			(x, y) = point
			Dx = x - self[0][0]
			Dy = y - self[0][1]
			dx = self[1][0] - self[0][0]
			dy = self[1][1] - self[0][1]
			length = sqrt(dx*dx + dy*dy)
			A = (Dx*dx + Dy*dy) / length
			B = (Dy*dx - Dx*dy) / length
			return -self.options.epsilon < A < length + self.options.epsilon and -self.options.epsilon < B < self.options.epsilon
		except ZeroDivisionError:
			return False
	
	def hide(self, hide=False):
		for drawn in self.drawn:
			self.canvas.itemconfig(drawn, state='hidden' if hide else 'normal')
	
	def free_sides(self):
		return 2-len(self.in_triangles)-(1 if self.equivalent_edge is not None else 0)
	
	def is_internal(self):
		return len(self.in_triangles) == 2
	
	def flip_orientation(self):
		self.vertices = self.vertices[::-1]
		self.update()
	
	def update(self):
		m = ((1-ARROW_FRAC)*self.vertices[0][0] + ARROW_FRAC*self.vertices[1][0], (1-ARROW_FRAC)*self.vertices[0][1] + ARROW_FRAC*self.vertices[1][1])
		self.canvas.coords(self.drawn[0], *[c for v in [self.vertices[0], m] for c in v])
		self.canvas.coords(self.drawn[1], *[c for v in [self.vertices[0], self.vertices[1]] for c in v])
	
	def set_current_colour(self, colour=None):
		if colour is None: colour = self.colour
		for drawn in self.drawn:
			self.canvas.itemconfig(drawn, fill=colour)

class CanvasTriangle(DrawableObject):
	def __init__(self, canvas, edges, options):
		super(CanvasTriangle, self).__init__(canvas, list(set(v for e in edges for v in e)), options)
		self.default_colour = self.colour = DEFAULT_TRIANGLE_COLOUR
		self.edges = edges
		
		# We reorder the vertices to guarantee that the vertices are cyclically ordered anticlockwise in the plane.
		d10, d20 = self[1] - self[0], self[2] - self[0]
		if d10[0]*d20[1] - d10[1]*d20[0] > 0: self.vertices = [self[0], self[2], self[1]]
		# Now we reorder the edges such that edges[i] does not meet vertices[i].
		self.edges = [edge for vertex in self for edge in self.edges if vertex not in edge.vertices]
		
		# And check to make sure everyone made it through alive.
		assert(len(self.edges) == 3)
		assert(self[0] != self[1] and self[1] != self[2] and self[2] != self[0])
		assert(self.edges[0] != self.edges[1] and self.edges[1] != self.edges[2] and self.edges[2] != self.edges[0])
		
		self.drawn = self.canvas.create_polygon([c for v in self for c in v], fill=self.default_colour, tag='polygon')
		# Add this triangle to each edge involved.
		for edge in self.edges:
			edge.in_triangles.append(self)
	
	def __contains__(self, point):
		if any(point in vertex for vertex in self):
			return False
		if any(point in edge for edge in self.edges):
			return False
		
		v0 = self[2] - self[0]
		v1 = self[1] - self[0]
		v2 = (point[0] - self[0][0], point[1] - self[0][1])
		
		dot00 = dot(v0, v0)
		dot01 = dot(v0, v1)
		dot02 = dot(v0, v2)
		dot11 = dot(v1, v1)
		dot12 = dot(v1, v2)
		
		invDenom = 1.0 / (dot00 * dot11 - dot01 * dot01)
		u = (dot11 * dot02 - dot01 * dot12) * invDenom
		v = (dot00 * dot12 - dot01 * dot02) * invDenom
		
		return (u >= 0) and (v >= 0) and (u + v <= 1)

class CurveComponent(DrawableObject):
	def __init__(self, canvas, vertices, options, multiplicity=1, smooth=False):
		super(CurveComponent, self).__init__(canvas, vertices, options)
		self.default_colour = self.colour = DEFAULT_CURVE_COLOUR
		self.drawn = self.canvas.create_line(
			[c for v in self.vertices for c in v],
			width=self.options.line_size,
			fill=self.colour,
			tag='curve',
			smooth=smooth
			)
		self.multiplicity = multiplicity
	
	def append_point(self, point):
		self.vertices.append(point)
		self.update()
		return self
	
	def pop_point(self):
		if len(self.vertices) > 2:
			self.vertices.pop()
			self.update()
	
	def move_point(self, index, x, y):
		self.vertices[index] = (x, y)
		self.update()

class TrainTrackBlock(DrawableObject):
	def __init__(self, canvas, vertices, options, multiplicity=1, smooth=False):
		super(TrainTrackBlock, self).__init__(canvas, vertices, options)
		self.default_colour = self.colour = DEFAULT_TRAIN_TRACK_BLOCK_COLOUR
		self.drawn = self.canvas.create_polygon(
			[v[j] for v in self.vertices for j in range(2)],
			fill=self.default_colour,
			tag='train_track',
			outline=self.default_colour,
			smooth=smooth
			)
		self.multiplicity = multiplicity





''' A module for interfacing with Sage. '''

from __future__ import absolute_import

import flipper

from sage.all import Matrix, NumberField, lcm

def directed_eigenvector(action_matrix, condition_matrix):
	''' An implementation of flipper.kernel.symboliccomputation.directed_eigenvector() using Sage.
	
	See the docstring of the above function for further details. '''
	
	M = Matrix(action_matrix.rows)
	
	for polynomial, _ in M.characteristic_polynomial().factor():
		degree = int(polynomial.degree())
		
		if degree > 1:
			flipper_polynomial_roots = flipper.kernel.Polynomial([int(x) for x in polynomial.coefficients(sparse=False)]).real_roots()
			
			if len(flipper_polynomial_roots) > 0:
				# We need only consider the largest root as it has to be >=1 and bigger than all of its Galois conjugates.
				flipper_polynomial_root = max(flipper_polynomial_roots)
				
				if flipper_polynomial_root >= 1:
					# Compute the kernel:
					K = NumberField(polynomial, 'L')
					a = K.gen()
					
					kernel_basis = (M - a).right_kernel().basis()
					
					basis = [[entry.polynomial().coefficients(sparse=False) for entry in v] for v in kernel_basis]
					scale = lcm([int(coeff.denominator()) for v in basis for entry in v for coeff in entry])
					scaled_basis = [[[int(int(coeff.numerator()) * scale) / int(coeff.denominator()) for coeff in entry] for entry in v] for v in basis]
					
					N = flipper.kernel.NumberField(flipper_polynomial_root)
					flipper_basis_matrix = flipper.kernel.Matrix([[N.element(entry) for entry in v] for v in scaled_basis])
					
					if len(flipper_basis_matrix) == 1:  # If rank(kernel) == 1.
						[flipper_eigenvector] = flipper_basis_matrix
						if flipper.kernel.matrix.nonnegative(flipper_eigenvector) and condition_matrix.nonnegative_image(flipper_eigenvector):
							return N.lmbda, flipper_eigenvector
					else:
						# We could use sage.Polyhedron here.
						T = (flipper.kernel.id_matrix(condition_matrix.width).join(condition_matrix)) * flipper_basis_matrix.transpose()
						try:
							linear_combination = T.find_vector_with_nonnegative_image()
							return N.lmbda, flipper_basis_matrix.transpose()(linear_combination)
						except flipper.AssumptionError:  # Eigenspace is disjoint from the cone.
							pass
	
	raise flipper.ComputationError('No interesting eigenvalues in cell.')





''' The main window of the flipper GUI application. '''

import flipper
import flipper.application

import re
import os
import sys
import pickle
from math import sin, cos, pi, ceil, sqrt
from itertools import combinations
from decimal import Decimal

try:
	import Tkinter as TK
	import tkFileDialog
	import tkMessageBox
except ImportError:  # Python 3.
	try:
		import tkinter as TK
		import tkinter.filedialog as tkFileDialog
		import tkinter.messagebox as tkMessageBox
	except ImportError:
		raise ImportError('Tkinter not available.')

try:
	import ttk as TTK
except ImportError:  # Python 3.
	try:
		from tkinter import ttk as TTK
	except ImportError:
		raise ImportError('Ttk not available.')

# Some constants.
if sys.platform in ['darwin']:
	COMMAND = {
		'new': 'Command+N',
		'open': 'Command+O',
		'save': 'Command+S',
		'close': 'Command+W',
		'lamination': 'Command+L',
		'erase': 'Command+D',
		'twist': 'Command+T',
		'halftwist': 'Command+H',
		'isometry': 'Command+I',
		'compose': 'Command+M'
		}
	COMMAND_KEY = {
		'new': '<Command-n>',
		'open': '<Command-o>',
		'save': '<Command-s>',
		'close': '<Command-w>',
		'lamination': '<Command-l>',
		'erase': '<Command-d>',
		'twist': '<Command-t>',
		'halftwist': '<Command-h>',
		'isometry': '<Command-i>',
		'compose': '<Command-m>'
		}
else:
	COMMAND = {
		'new': 'Ctrl+N',
		'open': 'Ctrl+O',
		'save': 'Ctrl+S',
		'close': 'Ctrl+W',
		'lamination': 'Ctrl+L',
		'erase': 'Ctrl+D',
		'twist': 'Ctrl+T',
		'halftwist': 'Ctrl+H',
		'isometry': 'Ctrl+I',
		'compose': 'Ctrl+M'
		}
	COMMAND_KEY = {
		'new': '<Control-n>',
		'open': '<Control-o>',
		'save': '<Control-s>',
		'close': '<Control-w>',
		'lamination': '<Control-l>',
		'erase': '<Control-d>',
		'twist': '<Control-t>',
		'halftwist': '<Control-h>',
		'isometry': '<Control-i>',
		'compose': '<Control-m>'
		}

# Regexs for validating names of things.
VALID_NAME_REGEX = r'[a-zA-Z][a-zA-z0-9_^-]*$'  # Valid names consist of letters, numbers, underscores and start with a letter.
VALID_SPECIFICATION_REGEX = r'[a-zA-Z]+$'  # Valid specifications are non-empty and consists of letters.
VALID_ISOMETRY_REGEX = r'([0-9]+:[0-9]+( |$))+'  # Valid isometries match 'num:num num:num ...'.

# Event modifier keys. Originate from: http://effbot.org/tkinterbook/tkinter-events-and-bindings.htm
BIT_SHIFT = 0x001
# BIT_CAPSLOCK = 0x002
# BIT_CONTROL = 0x004
# BIT_LEFT_ALT = 0x008
# BIT_NUMLOCK = 0x010
# BIT_RIGHT_ALT = 0x080
# BIT_MB_1 = 0x100
# BIT_MB_2 = 0x200
# BIT_MB_3 = 0x400

# Vectors to offset a label by to produce backing.
OFFSETS = [(1.5*cos(2 * pi * i / 12), 1.5*sin(2 * pi * i / 12)) for i in range(12)]

# Colours of things.
DEFAULT_EDGE_LABEL_COLOUR = 'black'
DEFAULT_EDGE_LABEL_BG_COLOUR = 'white'
DEFAULT_SELECTED_COLOUR = 'red'
MAX_DRAWABLE = 1000  # Maximum weight of a multicurve to draw fully.

def dot(a, b):
	return a[0] * b[0] + a[1] * b[1]

def helper(glob, method, args):
	result = method(*args)
	
	return glob, result

class FlipperApplication(object):
	def __init__(self, parent):
		self.parent = parent
		self.options = flipper.application.Options(self)
		self.colour_picker = flipper.application.ColourPalette()
		
		self.panels = TK.PanedWindow(self.parent, orient='horizontal', relief='raised')
		
		self.frame_interface = TK.Frame(self.parent)
		###
		TTK.Style().configure('Treeview', font=self.options.application_font)
		self.treeview_objects = TTK.Treeview(self.frame_interface, selectmode='browse')
		self.treeview_objects.heading('#0', text='Objects:', anchor='w')
		self.scrollbar_treeview = TK.Scrollbar(self.frame_interface, orient='vertical', command=self.treeview_objects.yview)
		self.treeview_objects.configure(yscroll=self.scrollbar_treeview.set)
		self.treeview_objects.bind('<Button-1>', self.treeview_objects_left_click)
		self.treeview_objects.bind('<Double-Button-1>', self.treeview_objects_double_left_click)
		self.treeview_objects.tag_configure('txt', font=self.options.application_font)
		self.treeview_objects.tag_configure('Heading', font=self.options.application_font)
		self.treeview_objects.insert('', 'end', 'triangulation', text='Triangulation: Incomplete', open=True, tags=['txt', 'menu'])
		self.treeview_objects.insert('', 'end', 'laminations', text='Laminations:', open=True, tags=['txt', 'menu'])
		self.treeview_objects.insert('', 'end', 'mapping_classes', text='Mapping Classes:', open=True, tags=['txt', 'menu'])
		
		self.treeview_objects.grid(row=0, column=0, sticky='nesw')
		self.scrollbar_treeview.grid(row=0, column=1, sticky='nws')
		self.frame_interface.grid_rowconfigure(0, weight=1)
		self.frame_interface.grid_columnconfigure(0, weight=1)
		###
		
		self.frame_draw = TK.Frame(self.parent)
		###
		# This needs takefocus set so that we can tell if it has been selected.
		# Also, for some reason which I can't explain, we need this height=1 to prevent the command
		# bar below from collapsing when the application is small.
		self.canvas = TK.Canvas(self.frame_draw, height=1, bg='#dcecff', takefocus=True)
		self.canvas.pack(padx=6, pady=6, fill='both', expand=True)
		self.canvas.bind('<Button-1>', self.canvas_left_click)
		self.canvas.bind('<Button-3>', self.canvas_right_click)
		self.canvas.bind('<Motion>', self.canvas_move)
		self.canvas.bind('<FocusOut>', self.canvas_focus_lost)
		
		self.frame_command = TK.Frame(self.parent)
		###
		
		self.panels.add(self.frame_interface, width=260)  # Make sure to set an inital width.
		self.panels.add(self.frame_draw)
		self.panels.pack(fill='both', expand=True)
		self.frame_command.pack(fill='x', expand=False)
		
		###
		
		# Create the menus.
		# Make sure to start the Lamination and Mapping class menus disabled.
		self.menubar = TK.Menu(self.parent)
		app_font = self.options.application_font  # Get a shorter name.
		
		self.filemenu = TK.Menu(self.menubar, tearoff=0)
		self.filemenu.add_command(label='New', command=self.initialise, accelerator=COMMAND['new'])
		self.filemenu.add_command(label='Open...', command=self.load, accelerator=COMMAND['open'])
		self.filemenu.add_command(label='Open example...', command=self.load_example)
		self.filemenu.add_command(label='Save...', command=self.save, accelerator=COMMAND['save'])
		self.exportmenu = TK.Menu(self.menubar, tearoff=0)
		self.exportmenu.add_command(label='Export image...', command=self.export_image)
		self.exportmenu.add_command(label='Export kernel file...', command=self.export_kernel_file)
		self.filemenu.add_cascade(label='Export', menu=self.exportmenu)
		self.filemenu.add_separator()
		self.filemenu.add_command(label='Exit', command=self.quit, accelerator=COMMAND['close'])
		self.menubar.add_cascade(label='File', menu=self.filemenu)
		
		self.editmenu = TK.Menu(self.menubar, tearoff=0)
		# Add undo and redo here.
		#self.editmenu.add_command(label='Undo', command=self.undo, accelerator=COMMAND['undo'])
		#self.editmenu.add_command(label='Redo', command=self.redo, accelerator=COMMAND['redo'])
		#self.editmenu.add_separator()
		self.editmenu.add_command(label='Tighten lamination', command=self.tighten_lamination)
		self.editmenu.add_command(label='Erase lamination', command=self.destroy_lamination, accelerator=COMMAND['erase'])
		self.menubar.add_cascade(label='Edit ', menu=self.editmenu)
		# The additional " " here prevents Apple from treating this like a standard edit menu and automatically adding a
		# "Start Dictation..." and "Emoji & Symbols" entries.
		
		self.createmenu = TK.Menu(self.menubar, tearoff=0)
		self.createmenu.add_command(label='Lamination', command=self.store_lamination, accelerator=COMMAND['lamination'])
		self.mappingclassmenu = TK.Menu(self.menubar, tearoff=0)
		self.mappingclassmenu.add_command(label='Twist', command=self.store_twist, accelerator=COMMAND['twist'])
		self.mappingclassmenu.add_command(label='Half twist', command=self.store_halftwist, accelerator=COMMAND['halftwist'])
		self.mappingclassmenu.add_command(label='Isometry', command=self.store_isometry, accelerator=COMMAND['isometry'])
		self.mappingclassmenu.add_command(label='Composition', command=self.store_composition, accelerator=COMMAND['compose'])
		self.createmenu.add_cascade(label='Mapping class', menu=self.mappingclassmenu)  # state='disabled')
		self.menubar.add_cascade(label='Create', menu=self.createmenu)
		
		
		##########################################
		self.settingsmenu = TK.Menu(self.menubar, tearoff=0)
		
		self.sizemenu = TK.Menu(self.menubar, tearoff=0)
		self.sizemenu.add_radiobutton(label='Small', var=self.options.size_var, value=flipper.application.options.SIZE_SMALL)
		self.sizemenu.add_radiobutton(label='Medium', var=self.options.size_var, value=flipper.application.options.SIZE_MEDIUM)
		self.sizemenu.add_radiobutton(label='Large', var=self.options.size_var, value=flipper.application.options.SIZE_LARGE)
		# self.sizemenu.add_radiobutton(label='Extra large', var=self.options.size_var, value=flipper.application.options.SIZE_XLARGE)
		
		self.edgelabelmenu = TK.Menu(self.menubar, tearoff=0)
		self.edgelabelmenu.add_radiobutton(label=flipper.application.options.LABEL_EDGES_NONE, var=self.options.label_edges_var)
		self.edgelabelmenu.add_radiobutton(label=flipper.application.options.LABEL_EDGES_INDEX, var=self.options.label_edges_var)
		self.edgelabelmenu.add_radiobutton(label=flipper.application.options.LABEL_EDGES_GEOMETRIC, var=self.options.label_edges_var)
		self.edgelabelmenu.add_radiobutton(label=flipper.application.options.LABEL_EDGES_ALGEBRAIC, var=self.options.label_edges_var)
		self.edgelabelmenu.add_separator()
		self.edgelabelmenu.add_checkbutton(label='Projectivise', var=self.options.projectivise_var)
		
		self.laminationdrawmenu = TK.Menu(self.menubar, tearoff=0)
		self.laminationdrawmenu.add_radiobutton(label=flipper.application.options.RENDER_LAMINATION_FULL, var=self.options.render_lamination_var)
		self.laminationdrawmenu.add_radiobutton(label=flipper.application.options.RENDER_LAMINATION_C_TRAIN_TRACK, var=self.options.render_lamination_var)
		self.laminationdrawmenu.add_radiobutton(label=flipper.application.options.RENDER_LAMINATION_W_TRAIN_TRACK, var=self.options.render_lamination_var)
		self.laminationdrawmenu.add_separator()
		self.laminationdrawmenu.add_checkbutton(label='Draw laminations straight', var=self.options.straight_laminations_var)
		
		self.zoommenu = TK.Menu(self.menubar, tearoff=0)
		self.zoommenu.add_command(label='Zoom in', command=self.zoom_in, accelerator='+')
		self.zoommenu.add_command(label='Zoom out', command=self.zoom_out, accelerator='-')
		self.zoommenu.add_command(label='Zoom to drawing', command=self.zoom_to_drawing, accelerator='0')
		
		self.settingsmenu.add_cascade(label='Sizes', menu=self.sizemenu)
		self.settingsmenu.add_cascade(label='Edge label', menu=self.edgelabelmenu)
		self.settingsmenu.add_cascade(label='Draw lamination', menu=self.laminationdrawmenu)
		self.settingsmenu.add_cascade(label='Zoom', menu=self.zoommenu)
		self.settingsmenu.add_checkbutton(label='Show internal edges', var=self.options.show_internals_var)
		self.settingsmenu.add_checkbutton(label='Show edge orientations', var=self.options.show_orientations_var)
		
		self.menubar.add_cascade(label='Settings', menu=self.settingsmenu)
		
		self.helpmenu = TK.Menu(self.menubar, tearoff=0)
		self.helpmenu.add_command(label='Help', command=self.show_help, accelerator='F1')
		self.helpmenu.add_separator()
		self.helpmenu.add_command(label='About', command=self.show_about)
		
		self.menubar.add_cascade(label='Help', menu=self.helpmenu)
		self.parent.config(menu=self.menubar)
		
		self.parent.bind(COMMAND_KEY['new'], lambda event: self.initialise())
		self.parent.bind(COMMAND_KEY['open'], lambda event: self.load())
		self.parent.bind(COMMAND_KEY['save'], lambda event: self.save())
		self.parent.bind(COMMAND_KEY['close'], lambda event: self.quit())
		self.parent.bind(COMMAND_KEY['lamination'], lambda event: self.store_lamination())
		self.parent.bind(COMMAND_KEY['erase'], lambda event: self.destroy_lamination())
		self.parent.bind(COMMAND_KEY['twist'], lambda event: self.store_twist())
		self.parent.bind(COMMAND_KEY['halftwist'], lambda event: self.store_halftwist())
		self.parent.bind(COMMAND_KEY['isometry'], lambda event: self.store_isometry())
		self.parent.bind(COMMAND_KEY['compose'], lambda event: self.store_composition())
		self.parent.bind('<Key>', self.parent_key_press)
		
		self.parent.protocol('WM_DELETE_WINDOW', self.quit)
		
		self.unsaved_work = False
		
		self.vertices = []
		self.edges = []
		self.triangles = []
		self.curve_components = []
		self.train_track_blocks = []
		
		self.zeta = 0
		self.equipped_triangulation = None
		self.current_lamination = None
		self.lamination_names = {}
		self.treeview_laminations = []
		self.mapping_class_names = {}
		self.treeview_mapping_classes = []
		
		self.selected_object = None
		self.output = None
	
	def initialise(self):
		if self.unsaved_work:
			result = tkMessageBox.showwarning('Unsaved work', 'Save before unsaved work is lost?', type='yesnocancel')
			if (result == 'yes' and not self.save()) or result == 'cancel':
				return False
		
		self.select_object(None)
		self.destroy_all_vertices()
		self.colour_picker.reset()
		
		self.unsaved_work = False
		return True
	
	def valid_name(self, strn):
		if re.match(VALID_NAME_REGEX, strn) is None:
			tkMessageBox.showerror('Invalid name', 'A valid name must match "%s".' % VALID_NAME_REGEX)
			return False
		
		return True
	
	def valid_specification(self, strn):
		if re.match(VALID_SPECIFICATION_REGEX, strn) is None:
			tkMessageBox.showerror('Invalid specification', 'A valid specification must match "%s".' % VALID_SPECIFICATION_REGEX)
			return False
		
		return True
	
	def valid_braid_specification(self, strn):
		try:
			n = int(strn)
			return n >= 3
		except ValueError:
			tkMessageBox.showerror('Invalid specification', 'A valid specification must be an integer at least three.')
			return False
	
	def valid_isometry(self, strn):
		if re.match(VALID_ISOMETRY_REGEX, strn) is None:
			tkMessageBox.showerror('Invalid isometry specification', 'A valid specification must match "%s".' % VALID_ISOMETRY_REGEX)
			return False
		
		return True
	
	def valid_composition(self, strn):
		# A composition is valid if it is a list of mapping class names and inverse names separated by periods.
		try:
			self.equipped_triangulation.mapping_class(strn)
		except TypeError:
			tkMessageBox.showerror('Invalid composition', 'A valid composition must consist of mapping class names and inverse names, with periods to separate ambiguities.')
			return False
		
		return True
	
	def add_lamination(self, lamination, name):
		if self.remove_lamination(name):
			self.equipped_triangulation.laminations[name] = lamination
			iid = self.treeview_objects.insert('laminations', 'end', text=name, tags=['txt', 'lamination', 'show_lamination'])
			self.lamination_names[iid] = name
			
			self.treeview_laminations.append(name)
			
			self.unsaved_work = True
	
	def remove_lamination(self, name):
		if name in self.treeview_laminations:
			if tkMessageBox.showwarning('Remove lamination', 'Remove existing lamination %s?' % name, type='yesno') == 'yes':
				self.equipped_triangulation.laminations.pop(name)
				[entry] = [child for child in self.treeview_objects.get_children('laminations') if self.lamination_names[child] == name]
				self.treeview_objects.delete(entry)
				self.lamination_names = dict((iid, l_name) for iid, l_name in self.lamination_names.items() if l_name != name)
				self.treeview_laminations.remove(name)
			else:
				return False
		
		return True
	
	def add_mapping_class(self, mapping_class, name):
		name_inverse = name.swapcase()
		if self.remove_mapping_class(name) and self.remove_mapping_class(name_inverse):
			self.equipped_triangulation.pos_mapping_classes[name] = mapping_class
			self.equipped_triangulation.neg_mapping_classes[name_inverse] = mapping_class.inverse()
			self.equipped_triangulation.mapping_classes[name] = mapping_class
			self.equipped_triangulation.mapping_classes[name_inverse] = mapping_class.inverse()
			
			iid = self.treeview_objects.insert('mapping_classes', 'end', text=name, tags=['txt', 'mapping_class'])
			self.mapping_class_names[iid] = name
			order = mapping_class.order()
			order_string = 'Infinite' if order == 0 else str(order)
			type_string = '?' if order == 0 else 'Periodic'
			
			# Set up all the properties to appear under this label.
			# We will also set up self.mapping_class_names to point each item under this to <name> too.
			tagged_actions = [
				('Apply', 'apply_mapping_class'),
				('Apply inverse', 'apply_mapping_class_inverse'),
				]
			for label, tag in tagged_actions:
				self.mapping_class_names[self.treeview_objects.insert(iid, 'end', text=label, tags=['txt', tag])] = name
			
			iid_properties = self.treeview_objects.insert(iid, 'end', text='Properties', tags=['txt', 'properties'])
			tagged_properties = [
				('Order: %s' % order_string, 'mapping_class_order'),
				('Type: %s' % type_string, 'mapping_class_type'),
				('Invariant lamination...', 'mapping_class_invariant_lamination'),
				('Conjugate to...', 'mapping_class_conjugate'),
				('Bundle...', 'mapping_class_bundle')
				]
			for label, tag in tagged_properties:
				self.mapping_class_names[self.treeview_objects.insert(iid_properties, 'end', text=label, tags=['txt', tag])] = name
			
			self.treeview_mapping_classes.append(name)
			
			self.unsaved_work = True
	
	def remove_mapping_class(self, name):
		name_inverse = name.swapcase()
		if name in self.treeview_mapping_classes:
			if tkMessageBox.showwarning('Remove mapping class', 'Remove existing mapping class %s?' % name, type='yesno') == 'yes':
				self.equipped_triangulation.pos_mapping_classes.pop(name)
				self.equipped_triangulation.neg_mapping_classes.pop(name_inverse)
				self.equipped_triangulation.mapping_classes.pop(name)
				self.equipped_triangulation.mapping_classes.pop(name_inverse)
				
				[entry] = [child for child in self.treeview_objects.get_children('mapping_classes') if self.mapping_class_names[child] == name]
				self.treeview_objects.delete(entry)
				self.mapping_class_names = dict((iid, m_name) for iid, m_name in self.mapping_class_names.items() if m_name != name)
				self.treeview_mapping_classes.remove(name)
			else:
				return False
		
		return True
	
	def save(self):
		path = tkFileDialog.asksaveasfilename(defaultextension='.flp', filetypes=[('flipper files', '.flp'), ('all files', '.*')], title='Save Flipper File')
		if path != '':
			try:
				spec = 'A flipper file.'
				version = flipper.__version__
				vertices = [(vertex[0], vertex[1]) for vertex in self.vertices]
				edges = [(self.vertices.index(edge[0]), self.vertices.index(edge[1]), edge.index, self.edges.index(edge.equivalent_edge) if edge.equivalent_edge is not None else None) for edge in self.edges]
				equipped_triangulation = self.equipped_triangulation
				canvas_objects = (vertices, edges)
				data = (equipped_triangulation, canvas_objects)
				
				pickled_objects = pickle.dumps((spec, version, data))
				open(path, 'wb').write(pickled_objects)
				self.unsaved_work = False
				return True
			except IOError:
				tkMessageBox.showwarning('Save Error', 'Could not open: %s' % path)
		
		return False
	
	def load(self, load_from=None):
		''' Load up some information.
		
		We can load from:
			- the path to a flipper file,
			- the contents of flipper file, or
			- something that flipper.kernel.package can eat.
		If given nothing it asks the user to select a flipper (kernel) file.'''
		
		try:
			if load_from is None or isinstance(load_from, (flipper.kernel.FileType, flipper.StringType)):
				if load_from is None:
					file_path = tkFileDialog.askopenfilename(
						defaultextension='.flp',
						filetypes=[('flipper files', '.flp'), ('all files', '.*')],
						title='Open flipper File')
					if not file_path:  # Cancelled the dialog.
						return
					try:
						string_contents = open(file_path, 'rb').read()
					except IOError:
						raise flipper.AssumptionError('Error 101: Cannot read contents of %s.' % load_from)
				elif isinstance(load_from, flipper.kernel.FileType):
					string_contents = load_from.read()
				elif isinstance(load_from, flipper.StringType):
					try:
						string_contents = open(load_from, 'rb').read()
					except IOError:
						string_contents = load_from
				
				try:
					spec, version, data = pickle.loads(string_contents)
				except (EOFError, AttributeError, KeyError):
					raise flipper.AssumptionError('Error 103: Cannot depickle information provided.')
				except ValueError:
					raise flipper.AssumptionError('Error 104: Invalid depickle.')
				
				if version != flipper.__version__:
					raise flipper.AssumptionError('Error 105: This file was created in an older version of flipper (%s)' % version)
				
				if spec != 'A flipper file.':
					raise flipper.AssumptionError('Error 108: Invalid specification.')
				
				try:
					equipped_triangulation, (vertices, edges) = data
				except ValueError:
					raise flipper.AssumptionError('Error 106: Invalid depickle.')
			else:
				if isinstance(load_from, flipper.kernel.EquippedTriangulation):
					equipped_triangulation = load_from
				else:
					try:
						# Creating an equippedTriangulation can raise a ValueError in a lot of different ways.
						equipped_triangulation = flipper.kernel.create_equipped_triangulation(load_from)
					except ValueError as error:
						raise flipper.AssumptionError('Error 102: Cannot package the given data:\n %s.' % error.message)
				
				triangulation = equipped_triangulation.triangulation
				# We don't have any vertices or edges, so we'll create a triangulation ourselves.
				vertices, edges = [], []
				
				# Get a dual tree.
				_, dual_tree = triangulation.tree_and_dual_tree()
				components = triangulation.components()
				num_components = len(components)
				# Make sure we get the right sizes of things.
				self.parent.update_idletasks()
				w = int(self.canvas.winfo_width())
				h = int(self.canvas.winfo_height())
				
				# We will layout the components in a p x q grid.
				# Aim to maximise r === min(w / p, h / q) subject to pq >= num_components.
				# Note that there is probably a closed formula for the optimal value of p (and so q).
				p = max(range(1, num_components+1), key=lambda p: min(w / p, h / ceil(float(num_components) / p)))
				q = int(ceil(float(num_components) / p))
				
				r = min(w / p, h / q) * (1 + self.options.zoom_fraction) / 4
				dx = w / p
				dy = h / q
				
				num_used_vertices = 0
				for index, component in enumerate(components):
					# Get the number of triangles.
					n = len(component) // 3  # Remember component double counts edges.
					ngon = n + 2
					
					# Create the vertices.
					for i in range(ngon):
						vertices.append((
							dx * (index % p) + dx / 2 + r * sin(2 * pi * (i + 0.5) / ngon),
							dy * int(index / p) + dy / 2 + r * cos(2 * pi * (i + 0.5) / ngon)
							))
					
					def num_descendants(edge_label):
						''' Return the number of triangles that can be reached in the dual tree starting at the given edge_label. '''
						
						corner = triangulation.corner_of_edge(edge_label)
						left = (1 + sum(num_descendants(~(corner.labels[2])))) if dual_tree[corner.indices[2]] else 0
						right = (1 + sum(num_descendants(~(corner.labels[1])))) if dual_tree[corner.indices[1]] else 0
						
						return left, right
					
					initial_edge_index = min(i for i in component if i >= 0 and not dual_tree[i])
					to_extend = [(num_used_vertices, num_used_vertices+1, initial_edge_index)]
					# Hmmm, need to be more careful here to ensure that we correctly orient the edges.
					edges.append((num_used_vertices+1, num_used_vertices+0, initial_edge_index, None))
					while to_extend:
						source_vertex, target_vertex, label = to_extend.pop()
						left, right = num_descendants(label)
						far_vertex = target_vertex + left + 1
						corner = triangulation.corner_of_edge(label)
						
						if corner.labels[2] == corner.indices[2]:
							edges.append((far_vertex, target_vertex, corner.indices[2], None))
						else:
							edges.append((target_vertex, far_vertex, corner.indices[2], None))
						if corner.labels[1] == corner.indices[1]:
							edges.append((source_vertex, far_vertex, corner.indices[1], None))
						else:
							edges.append((far_vertex, source_vertex, corner.indices[1], None))
						
						if left > 0:
							to_extend.append((far_vertex, target_vertex, ~(corner.labels[2])))
						
						if right > 0:
							to_extend.append((source_vertex, far_vertex, ~(corner.labels[1])))
					num_used_vertices = len(vertices)
				
				# Glue together sides with the same index.
				for i, j in combinations(range(len(edges)), r=2):
					if edges[i][2] == edges[j][2]:
						edges[i] = (edges[i][0], edges[i][1], edges[i][2], j)
						edges[j] = (edges[j][0], edges[j][1], edges[j][2], i)
			
			if not self.initialise():
				return
			
			# Create the vertices.
			for vertex in vertices:
				self.create_vertex(vertex)
			
			# Create the edges.
			for edge in edges:
				start_index, end_index, edge_index, glued_to_index = edge
				self.create_edge(self.vertices[start_index], self.vertices[end_index])
			
			# Create the edge identifications.
			for index, edge in enumerate(edges):
				start_index, end_index, edge_index, glued_to_index = edge
				if glued_to_index is not None and glued_to_index > index:
					self.create_edge_identification(self.edges[index], self.edges[glued_to_index])
			
			# Set the correct edge indices.
			for index, edge in enumerate(edges):
				start_index, end_index, edge_index, glued_to_index = edge
				self.edges[index].index = edge_index
			
			self.equipped_triangulation = equipped_triangulation
			
			self.zoom_to_drawing()
			
			
			for name, lamination in sorted(self.equipped_triangulation.laminations.items(),
				key=lambda x: (len(x[0]), x[0])):
				self.add_lamination(lamination, name)
			
			for name, mapping_class in sorted(self.equipped_triangulation.pos_mapping_classes.items(),
				key=lambda x: (len(x[0]), x[0])):
				self.add_mapping_class(mapping_class, name)
			
			# Get the correct empty lamination.
			self.destroy_lamination()
			
			self.unsaved_work = False
		except (flipper.AssumptionError, IndexError, ValueError) as error:
			tkMessageBox.showerror('Load Error', error.message)
	
	def load_example(self):
		example = flipper.application.get_choice('Open example.', 'Choose example to open.', [
			'Circular n-gon',
			'Radial n-gon',
			'From isomorphism signature',
			'Sperical braid',
			'S_{0,4}',
			'S_{1,1}',
			'S_{1,2}',
			'S_{2,1}',
			'S_{3,1}',
			'S_{4,1}',
			'S_{5,1}'])
		if example == 'Circular n-gon':
			self.initialise_circular_n_gon()
		elif example == 'Radial n-gon':
			self.initialise_radial_n_gon()
		elif example == 'Sperical braid':
			strands = flipper.application.get_input('Spherical braid specification', 'Number of strands for spherical braid', validate=self.valid_braid_specification)
			if strands is not None:
				self.load(flipper.load('SB_%s' % strands))
		elif example == 'From isomorphism signature':
			signature = flipper.application.get_input('Triangulation specification', 'Isomorphism signature:')
			if signature is not None:
				try:
					self.load([flipper.triangulation_from_iso_sig(signature)])
				except AssertionError:
					tkMessageBox.showerror('Load Error', 'Invalid isomorphism signature.')
		elif example == 'S_{0,4}':
			self.load(flipper.load('S_0_4'))
		elif example == 'S_{1,1}':
			self.load(flipper.load('S_1_1'))
		elif example == 'S_{1,2}':
			self.load(flipper.load('S_1_2'))
		elif example == 'S_{2,1}':
			self.load(flipper.load('S_2_1'))
		elif example == 'S_{3,1}':
			self.load(flipper.load('S_3_1'))
		elif example == 'S_{4,1}':
			self.load(flipper.load('S_4_1'))
		elif example == 'S_{5,1}':
			self.load(flipper.load('S_5_1'))
	
	def export_image(self):
		path = tkFileDialog.asksaveasfilename(defaultextension='.ps', filetypes=[('postscript files', '.ps'), ('all files', '.*')], title='Export Image')
		if path:
			try:
				self.canvas.postscript(file=path, colormode='color')
			except IOError:
				tkMessageBox.showwarning('Export Error', 'Could not open: %s' % path)
	
	def export_kernel_file(self):
		if self.is_complete():
			path = tkFileDialog.asksaveasfilename(defaultextension='.flp', filetypes=[('flipper kernel file', '.flp'), ('all files', '.*')], title='Export Kernel File')
			if path:
				try:
					example = flipper.kernel.package(self.equipped_triangulation)
					with open(path, 'wb') as disk_file:
						disk_file.write(example)
				except IOError:
					tkMessageBox.showwarning('Export Error', 'Could not open: %s' % path)
		else:
			tkMessageBox.showwarning('Export Error', 'Cannot export incomplete surface.')
	
	def quit(self):
		# Write down our current state for output. If we are incomplete then this is just None.
		self.output = self.equipped_triangulation
		
		if self.initialise():
			# Apparantly there are some problems with comboboxes, see:
			#  http://stackoverflow.com/questions/15448914/python-tkinter-ttk-combobox-throws-exception-on-quit
			self.parent.eval('::ttk::CancelRepeat')
			self.parent.destroy()
			self.parent.quit()
	
	def show_help(self):
		flipper.doc.open_documentation()
	
	def show_about(self):
		tkMessageBox.showinfo('About', 'flipper (Version %s).\nCopyright (c) Mark Bell 2013 - 2016.' % flipper.__version__)
	
	def translate(self, dx, dy):
		for vertex in self.vertices:
			vertex[0] = vertex[0] + dx
			vertex[1] = vertex[1] + dy
		
		for curve_component in self.curve_components + self.train_track_blocks:
			for i in range(len(curve_component.vertices)):
				curve_component.vertices[i] = curve_component.vertices[i][0] + dx, curve_component.vertices[i][1] + dy
		
		self.canvas.move('all', dx, dy)
	
	def zoom(self, scale):
		for vertex in self.vertices:
			vertex[0], vertex[1] = scale * vertex[0], scale * vertex[1]
			vertex.update()
		for edge in self.edges:
			edge.update()
		for triangle in self.triangles:
			triangle.update()
		for curve_component in self.curve_components + self.train_track_blocks:
			for i in range(len(curve_component.vertices)):
				curve_component.vertices[i] = scale * curve_component.vertices[i][0], scale * curve_component.vertices[i][1]
			curve_component.update()
		self.build_edge_labels()
		self.redraw()
	
	def zoom_in(self):
		self.zoom_centre(1.05)
	
	def zoom_out(self):
		self.zoom_centre(0.95)
	
	def zoom_centre(self, scale):
		self.parent.update_idletasks()
		cw = int(self.canvas.winfo_width())
		ch = int(self.canvas.winfo_height())
		self.translate(-cw / 2, -ch / 2)
		self.zoom(scale)
		self.translate(cw / 2, ch / 2)
	
	def zoom_to_drawing(self):
		self.parent.update_idletasks()
		box = self.canvas.bbox('all')
		if box is not None:
			x0, y0, x1, y1 = box
			cw = int(self.canvas.winfo_width())
			ch = int(self.canvas.winfo_height())
			cr = min(cw, ch)
			
			w, h = x1 - x0, y1 - y0
			r = max(w, h)
			
			self.translate(-x0 - w / 2, -y0 - h / 2)
			self.zoom(self.options.zoom_fraction * float(cr) / r)
			self.translate(cw / 2, ch / 2)
	
	def is_complete(self):
		return len(self.triangles) > 0 and all(edge.free_sides() == 0 for edge in self.edges)
	
	def object_here(self, point):
		for piece in self.vertices + self.edges + self.triangles:
			if point in piece:
				return piece
		return None
	
	def redraw(self):
		self.build_edge_labels()
		
		for vertex in self.vertices:
			vertex.update()
		self.canvas.itemconfig('line', width=self.options.line_size)
		self.canvas.itemconfig('curve', width=self.options.line_size)
		# Only put arrows on the start so arrow heads appear in the middle.
		self.canvas.itemconfig('line_start', arrow='last' if self.options.show_orientations else '')
		self.canvas.itemconfig('line', arrowshape=self.options.arrow_shape)
		
		for edge in self.edges:
			edge.hide(not self.options.show_internals and edge.is_internal())
		self.canvas.tag_raise('polygon')
		self.canvas.tag_raise('line')
		self.canvas.tag_raise('oval')
		self.canvas.tag_raise('train_track')
		self.canvas.tag_raise('curve')
		self.canvas.tag_raise('label')
		self.canvas.tag_raise('edge_label')
	
	def select_object(self, selected_object):
		# We can only ever select vertices, edges and curve_components.
		assert(selected_object is None or \
			isinstance(selected_object, (flipper.application.CanvasVertex, flipper.application.CanvasEdge, flipper.application.CurveComponent)))
		self.selected_object = selected_object
		for x in self.vertices + self.edges + self.curve_components + self.train_track_blocks:
			x.set_current_colour()
		if self.selected_object is not None:
			self.selected_object.set_current_colour(DEFAULT_SELECTED_COLOUR)
	
	
	######################################################################
	
	
	def initialise_radial_n_gon(self):
		gluing = flipper.application.get_input('Surface specification', 'Boundary pattern for radial ngon:', validate=self.valid_specification)
		if gluing is not None:
			if self.initialise():
				n = len(gluing)
				self.parent.update_idletasks()
				w = int(self.canvas.winfo_width())
				h = int(self.canvas.winfo_height())
				r = min(w, h)
				
				self.create_vertex((w / 2, h / 2))
				for i in range(n):
					self.create_vertex((w / 2 + sin(2*pi*(i+0.5) / n) * r, h / 2 + cos(2*pi*(i+0.5) / n) * r))
				
				for i in range(1, n):
					self.create_edge(self.vertices[i], self.vertices[i+1])
				self.create_edge(self.vertices[n], self.vertices[1])
				for i in range(n):
					self.create_edge(self.vertices[0], self.vertices[i+1])
				
				for i, j in combinations(range(n), r=2):
					if gluing[i] == gluing[j].swapcase():
						self.create_edge_identification(self.edges[i], self.edges[j])
				
				self.zoom_to_drawing()
				
				self.unsaved_work = True
	
	def initialise_circular_n_gon(self):
		gluing = flipper.application.get_input('Surface specification', 'Boundary pattern for circular ngon:', validate=self.valid_specification)
		if gluing is not None:
			if self.initialise():
				n = len(gluing)
				self.parent.update_idletasks()
				w = int(self.canvas.winfo_width())
				h = int(self.canvas.winfo_height())
				r = min(w, h) * (1 + self.options.zoom_fraction) / 4
				
				for i in range(n):
					self.create_vertex((w / 2 + sin(2*pi*(i+0.5) / n) * r, h / 2 + cos(2*pi*(i+0.5) / n) * r))
				for i in range(n):
					self.create_edge(self.vertices[i], self.vertices[i-1])
				
				all_vertices = list(range(n))
				while len(all_vertices) > 3:
					for i in range(0, len(all_vertices)-1, 2):
						self.create_edge(self.vertices[all_vertices[i]], self.vertices[all_vertices[(i+2) % len(all_vertices)]])
					all_vertices = all_vertices[::2]
				
				for i, j in combinations(range(n), r=2):
					if gluing[i] == gluing[j].swapcase():
						self.create_edge_identification(self.edges[i], self.edges[j])
				
				self.zoom_to_drawing()
				
				self.unsaved_work = True
	
	
	######################################################################
	
	
	def create_vertex(self, point):
		self.vertices.append(flipper.application.CanvasVertex(self.canvas, point, self.options))
		self.unsaved_work = True
		self.redraw()
		self.build_equipped_triangulation()
		return self.vertices[-1]
	
	def destroy_vertex(self, vertex=None):
		if vertex is None: vertex = self.vertices[-1]
		if self.selected_object == vertex: self.select_object(None)
		while True:
			for edge in self.edges:
				if edge[0] == vertex or edge[1] == vertex:
					self.destroy_edge(edge)
					break
			else:
				break
		self.canvas.delete(vertex.drawn)
		self.vertices.remove(vertex)
		self.unsaved_work = True
		self.redraw()
		self.build_equipped_triangulation()
	
	def destroy_all_vertices(self):
		while self.vertices:
			self.destroy_vertex()
		self.build_equipped_triangulation()
	
	def create_edge(self, v1, v2):
		# Check that the vertices are distinct
		if len(set([v1, v2])) != 2:
			return None
		
		# Check that this edge doesn't already exist.
		if any(set([edge[0], edge[1]]) == set([v1, v2]) for edge in self.edges):
			return None
		
		# Check that this edge doesn't intersect an existing one.
		if any(flipper.application.lines_intersect(edge[0], edge[1], v1, v2, self.options.float_error, True)[1] for edge in self.edges):
			return None
		
		e0 = flipper.application.CanvasEdge(self.canvas, [v1, v2], self.options)
		self.edges.append(e0)
		# Add in any needed triangles.
		for e1, e2 in combinations(self.edges, r=2):
			if e1 != e0 and e2 != e0:
				if e1.free_sides() > 0 and e2.free_sides() > 0:
					if len(set([e[0] for e in [e0, e1, e2]] + [e[1] for e in [e0, e1, e2]])) == 3:
						self.create_triangle(e0, e1, e2)
		self.unsaved_work = True
		self.redraw()
		self.build_equipped_triangulation()
		return self.edges[-1]
	
	def destroy_edge(self, edge=None):
		if edge is None: edge = self.edges[-1]
		if self.selected_object == edge: self.select_object(None)
		for drawn in edge.drawn:
			self.canvas.delete(drawn)
		for triangle in edge.in_triangles:
			self.destroy_triangle(triangle)
		self.destroy_edge_identification(edge)
		self.edges.remove(edge)
		self.unsaved_work = True
		self.build_equipped_triangulation()
		self.redraw()
	
	def create_triangle(self, e1, e2, e3):
		# Check that there are 3 edges.
		if len(set([e1, e2, e3])) != 3:
			return None
		
		# Check that this triangle doesn't already exist.
		if any([set(triangle.edges) == set([e1, e2, e3]) for triangle in self.triangles]):
			return None
		
		# Check that there are 3 vertices.
		corner_vertices = list(set(v for e in [e1, e2, e3] for v in e))
		if len(corner_vertices) != 3:
			return None
		
		# Check that there aren't any vertices inside the triangle.
		v0 = corner_vertices[2] - corner_vertices[0]
		v1 = corner_vertices[1] - corner_vertices[0]
		for vertex in self.vertices:
			if vertex not in corner_vertices:
				v2 = vertex - corner_vertices[0]
				
				dot00 = dot(v0, v0)
				dot01 = dot(v0, v1)
				dot02 = dot(v0, v2)
				dot11 = dot(v1, v1)
				dot12 = dot(v1, v2)
				
				invDenom = 1.0 / (dot00 * dot11 - dot01 * dot01)
				u = (dot11 * dot02 - dot01 * dot12) * invDenom
				v = (dot00 * dot12 - dot01 * dot02) * invDenom
				
				if (u >= 0) and (v >= 0) and (u + v <= 1):
					return None
		
		self.triangles.append(flipper.application.CanvasTriangle(self.canvas, [e1, e2, e3], self.options))
		
		self.unsaved_work = True
		self.redraw()
		self.build_equipped_triangulation()
		return self.triangles[-1]
	
	def destroy_triangle(self, triangle=None):
		if triangle is None: triangle = self.triangles[-1]
		self.canvas.delete(triangle.drawn)
		for edge in self.edges:
			if triangle in edge.in_triangles:
				edge.in_triangles.remove(triangle)
				self.destroy_edge_identification(edge)
		self.triangles.remove(triangle)
		self.unsaved_work = True
		self.redraw()
		self.build_equipped_triangulation()
	
	def create_edge_identification(self, e1, e2):
		if e1.equivalent_edge is not None or e2.equivalent_edge is not None:
			return None
		if e1.free_sides() != 1 or e2.free_sides() != 1:
			return None
		
		e1.equivalent_edge, e2.equivalent_edge = e2, e1
		# Now orient the edges so they match.
		[t1], [t2] = e1.in_triangles, e2.in_triangles
		[s1], [s2] = [i for i in range(3) if t1.edges[i] == e1], [i for i in range(3) if t2.edges[i] == e2]
		# Determine if the orientation of e1 (respectively e2) agrees with t1 (resp. t2).
		e1_agrees = e1[0] == t1[s1 + 1]
		e2_agrees = e2[0] == t2[s2 + 1]
		# We need one to agree and one to disagree - so if not then flip the orientation of e2.
		if e1_agrees == e2_agrees:
			e2.flip_orientation()
		
		# Change colour.
		new_colour = self.colour_picker.get_colour()
		e1.set_colour(new_colour)
		e2.set_colour(new_colour)
		self.unsaved_work = True
		self.build_equipped_triangulation()
	
	def destroy_edge_identification(self, edge):
		if edge.equivalent_edge is not None:
			other_edge = edge.equivalent_edge
			other_edge.set_colour()
			edge.set_colour()
			
			
			other_edge.set_colour(other_edge.default_colour)
			edge.set_colour(edge.default_colour)
			
			edge.equivalent_edge.equivalent_edge = None
			edge.equivalent_edge = None
			self.unsaved_work = True
		self.build_equipped_triangulation()
	
	def create_curve_component(self, vertices, multiplicity=1, smooth=False):
		self.curve_components.append(flipper.application.CurveComponent(self.canvas, vertices, self.options, multiplicity, smooth))
		return self.curve_components[-1]
	
	def destory_curve_component(self, curve_component):
		if self.selected_object == curve_component: self.select_object(None)
		self.canvas.delete(curve_component.drawn)
		self.curve_components.remove(curve_component)
	
	def create_train_track_block(self, vertices, multiplicity=1, smooth=False):
		self.train_track_blocks.append(flipper.application.TrainTrackBlock(self.canvas, vertices, self.options, multiplicity, smooth))
		return self.train_track_blocks[-1]
	
	def destroy_train_track_block(self, curve_component):
		self.canvas.delete(curve_component.drawn)
		self.train_track_blocks.remove(curve_component)
	
	def destroy_lamination(self):
		while self.curve_components != []:
			self.destory_curve_component(self.curve_components[-1])
		
		while self.train_track_blocks != []:
			self.destroy_train_track_block(self.train_track_blocks[-1])
		
		if self.is_complete():
			self.current_lamination = self.equipped_triangulation.triangulation.empty_lamination()
		
		self.select_object(None)
		self.redraw()
	
	
	######################################################################
	
	
	def set_edge_indices(self):
		# Assigns each edge an index in range(self.zeta).
		
		self.clear_edge_indices()
		self.zeta = 0
		for edge in self.edges:
			if edge.index == -1:
				self.zeta += 1
				edge.index = self.zeta-1
				if edge.equivalent_edge is not None:
					edge.equivalent_edge.index = edge.index
	
	def clear_edge_indices(self):
		self.zeta = 0
		for edge in self.edges:
			edge.index = -1
	
	def create_edge_labels(self):
		self.destroy_edge_labels()  # Remove existing labels.
		
		def accuracy_required(x):
			if isinstance(x, flipper.IntegerType):
				return flipper.kernel.height_int(x) + 1
			else:
				return x.height + x.log_degree
		def get_accurate(x, acc):
			if isinstance(x, flipper.IntegerType):
				return flipper.kernel.AlgebraicApproximation.from_int(x, acc)
			else:
				return x.algebraic_approximation(acc)
			
		
		# How to label the edge with given index.
		if self.options.label_edges == 'Index':
			labels = dict((index, index) for index in range(self.zeta))
		elif self.options.label_edges == 'Geometric':
			labels = dict((index, self.current_lamination(index)) for index in range(self.zeta))
		elif self.options.label_edges == 'Algebraic':
			labels = dict((index, self.current_lamination[index]) for index in range(self.zeta))
		elif self.options.label_edges == 'None':
			labels = dict((index, '') for index in range(self.zeta))
		else:
			raise ValueError()
		
		if self.options.projectivise and self.options.label_edges in ['Geometric', 'Algebraic']:
			accuracy_required = 2 * len(labels) * (max(accuracy_required(value) for value in labels.values()) + 1)
			labels = dict((index, get_accurate(labels[index], accuracy_required)) for index in labels)
			total = sum(labels[index] for index in range(self.zeta))
			if total != 0:  # There should probably be an else to this statement.
				# Note the "+ 0" to ensure that -0.0 appears as 0.0.
				labels = dict((index, round(float(labels[index] / total), 12) + 0) for index in range(self.zeta))
			else:
				labels = dict((index, round(float(labels[index]), 12) + 0) for index in range(self.zeta))
		
		for edge in self.edges:
			# We start by creating a nice background for the label. This ensures
			# that it is always readable, even when on top of a lamination.
			# To do this we first draw this label in a different colour with
			# slightly different offsets. This creates a nice 'bubble' effect
			# rather than having to draw a large bounding box.
			for offset in OFFSETS:
				self.canvas.create_text(
					[a+x for a, x in zip(edge.centre(), offset)],
					text=labels[edge.index],
					tag='edge_label',
					font=self.options.canvas_font,
					fill=DEFAULT_EDGE_LABEL_BG_COLOUR)
			
			self.canvas.create_text(edge.centre(),
				text=labels[edge.index],
				tag='edge_label',
				font=self.options.canvas_font,
				fill=DEFAULT_EDGE_LABEL_COLOUR)
	
	def destroy_edge_labels(self):
		self.canvas.delete('edge_label')
	
	def build_edge_labels(self):
		if self.is_complete():
			self.create_edge_labels()
		else:
			self.destroy_edge_labels()
	
	def create_equipped_triangulation(self):
		# Must start by calling self.set_edge_indices() so that self.zeta is correctly set.
		self.set_edge_indices()
		labels = [[triangle.edges[side].index if triangle[side+1] == triangle.edges[side][0] else ~triangle.edges[side].index for side in range(3)] for triangle in self.triangles]
		self.equipped_triangulation = flipper.kernel.EquippedTriangulation(flipper.create_triangulation(labels), [], [])
		triangulation = self.equipped_triangulation.triangulation
		self.current_lamination = self.equipped_triangulation.triangulation.empty_lamination()
		self.create_edge_labels()
		self.treeview_objects.item('triangulation', text='Triangulation:')
		self.treeview_objects.insert('triangulation', 'end', text='Genus: %d' % triangulation.genus, tags=['txt'])
		self.treeview_objects.insert('triangulation', 'end', text='Punctures: %d' % triangulation.num_vertices, tags=['txt'])
		self.treeview_objects.insert('triangulation', 'end', text='Euler characteristic: %d' % triangulation.euler_characteristic, tags=['txt'])
		#self.menubar.entryconfig('Lamination', state='normal')
		#self.menubar.entryconfig('Mapping class', state='normal')
	
	def destroy_equipped_triangulation(self):
		self.destroy_lamination()
		self.clear_edge_indices()
		self.destroy_edge_labels()
		self.equipped_triangulation = None
		self.current_lamination = None
		self.lamination_names = {}
		self.treeview_laminations = []
		self.mapping_class_names = {}
		self.treeview_mapping_classes = []
		for child in self.treeview_objects.get_children('laminations') + self.treeview_objects.get_children('mapping_classes'):
			self.treeview_objects.delete(child)
		self.treeview_objects.item('triangulation', text='Triangulation: Incomplete')
		self.treeview_objects.delete(*self.treeview_objects.get_children('triangulation'))
	
	def build_equipped_triangulation(self):
		if self.is_complete() and self.equipped_triangulation is None:
			self.create_equipped_triangulation()
		elif not self.is_complete() and self.equipped_triangulation is not None:
			self.destroy_equipped_triangulation()
	
	
	######################################################################
	
	
	def canvas_to_lamination(self):
		geometric = [0] * self.zeta
		
		# This version takes into account bigons between interior edges.
		for curve in self.curve_components:
			meets = []  # We store (index of edge intersection, should we double count).
			for i in range(len(curve.vertices)-1):
				this_segment_meets = [(flipper.application.lines_intersect(curve.vertices[i], curve.vertices[i+1], edge[0], edge[1], self.options.float_error, edge.equivalent_edge is None), edge.index) for edge in self.edges]
				for (d, double), index in sorted(this_segment_meets):
					if d >= -self.options.float_error:
						if len(meets) > 0 and meets[-1][0] == index:
							meets.pop()
						else:
							meets.append((index, double))
			
			for index, double in meets:
				geometric[index] += (2 if double else 1) * curve.multiplicity
		
		if all(isinstance(x, flipper.IntegerType) and x % 2 == 0 for x in geometric):
			geometric = [i // 2 for i in geometric]
		else:
			geometric = [i / 2 for i in geometric]
		
		self.current_lamination = self.equipped_triangulation.triangulation.lamination(geometric)
		
		return self.current_lamination
	
	def lamination_to_canvas(self, lamination):
		self.destroy_lamination()
		
		# Choose the right way to render this lamination.
		if not lamination.is_multicurve() or lamination.weight() > MAX_DRAWABLE:
			if self.options.render_lamination == flipper.application.options.RENDER_LAMINATION_FULL:
				render = flipper.application.options.RENDER_LAMINATION_W_TRAIN_TRACK
			else:
				render = self.options.render_lamination
		else:
			render = self.options.render_lamination
		
		# We'll do everything with floats now because these are accurate enough for drawing to the screen with.
		vb = self.options.vertex_buffer  # We are going to use this a lot.
		a_weights = [float(x) for x in lamination]
		master = float(max(a_weights))
		if master == 0: master = float(1)
		
		for triangle in self.triangles:
			a_tri_weights = [a_weights[edge.index] for edge in triangle.edges]
			a_dual_weights = [(a_tri_weights[(j+1)%3] + a_tri_weights[(j+2)%3] - a_tri_weights[(j+0)%3]) / 2 for j in range(3)]
			for i in range(3):
				a = triangle[i-1] - triangle[i]
				b = triangle[i-2] - triangle[i]
				
				if render == flipper.application.options.RENDER_LAMINATION_W_TRAIN_TRACK:
					if a_dual_weights[i] > 0.00001:  # Should be 0 but we have a floating point approximation.
						# We first do the edge to the left of the vertex.
						# Correction factor to take into account the weight on this edge.
						s_a = a_weights[triangle.edges[i-2].index] / master
						# The fractions of the distance of the two points on this edge.
						scale_a = vb * s_a + (1 - s_a) / 2
						scale_a2 = scale_a + (1 - 2*vb) * s_a * a_dual_weights[i] / (a_dual_weights[i] + a_dual_weights[i-1])
						
						# Now repeat for the other edge of the triangle.
						s_b = a_weights[triangle.edges[i-1].index] / master
						scale_b = vb * s_b + (1 - s_b) / 2
						scale_b2 = scale_b + (1 - 2*vb) * s_b * a_dual_weights[i] / (a_dual_weights[i] + a_dual_weights[i-2])
						
						S1, P1, Q1, E1 = flipper.application.interpolate(triangle[i-1], triangle[i], triangle[i-2], scale_a, scale_b)
						S2, P2, Q2, E2 = flipper.application.interpolate(triangle[i-1], triangle[i], triangle[i-2], scale_a2, scale_b2)
						if self.options.straight_laminations:
							self.create_train_track_block([S1, E1, E2, S2])
						else:
							self.create_train_track_block([S1, S1, P1, Q1, E1, E1, E2, E2, Q2, P2, S2, S2, S1, S1], smooth=True)
				elif render == flipper.application.options.RENDER_LAMINATION_FULL:  # We can ONLY use this method when the lamination is a multicurve.
					# Also it is VERY slow (O(n) not O(log(n))).
					# Here we need the exact dual weights so we had better work them out.
					weights = [lamination(edge.index) for edge in triangle.edges]
					wa, wb = weights[i-2], weights[i-1]
					dual_weights = [(weights[j-2] + weights[j-1] - weights[j]) // 2 for j in range(3)]
					for j in range(int(dual_weights[i])):
						scale_a = 0.5 if wa == 1 else vb + (1 - 2*vb) * ((wa - 1) * (master - wa) + 2 * wa * j) / (2 * (wa - 1) * master)
						scale_b = 0.5 if wb == 1 else vb + (1 - 2*vb) * ((wb - 1) * (master - wb) + 2 * wb * j) / (2 * (wb - 1) * master)
						
						S, P, Q, E = flipper.application.interpolate(triangle[i-1], triangle[i], triangle[i-2], scale_a, scale_b)
						if self.options.straight_laminations:
							self.create_curve_component([S, E])
						else:
							self.create_curve_component([S, P, Q, E], smooth=True)
				elif render == flipper.application.options.RENDER_LAMINATION_C_TRAIN_TRACK:
					if a_dual_weights[i] > 0:
						scale = 0.5
						S, P, Q, E = flipper.application.interpolate(triangle[i-1], triangle[i], triangle[i-2], scale, scale)
						if self.options.straight_laminations:
							self.create_curve_component([S, E])
						else:
							self.create_curve_component([S, P, Q, E], smooth=True)
		
		self.current_lamination = lamination
		self.create_edge_labels()
	
	def tighten_lamination(self):
		if self.is_complete():
			self.lamination_to_canvas(self.current_lamination)
	
	def store_lamination(self):
		if self.is_complete():
			name = flipper.application.get_input('Name', 'New lamination name:', validate=self.valid_name)
			if name is not None:
				self.add_lamination(self.current_lamination, name)
		else:
			tkMessageBox.showwarning('Incomplete triangulation', 'Cannot store lamination when triangulation is incomplete.')
	
	def store_twist(self):
		if self.is_complete():
			lamination = self.current_lamination
			
			if lamination.is_twistable():
				name = flipper.application.get_input('Name', 'New twist name:', validate=self.valid_name)
				if name is not None:
					self.add_mapping_class(lamination.encode_twist(), name)
			else:
				tkMessageBox.showwarning('Curve', 'Cannot twist about this, it is not a curve with punctured complementary regions.')
		else:
			tkMessageBox.showwarning('Incomplete triangulation', 'Cannot compute twist when triangulation is incomplete.')
	
	def store_halftwist(self):
		if self.is_complete():
			lamination = self.current_lamination
			
			if lamination.is_halftwistable():
				name = flipper.application.get_input('Name', 'New half twist name:', validate=self.valid_name)
				if name is not None:
					self.add_mapping_class(lamination.encode_halftwist(), name)
			else:
				tkMessageBox.showwarning('Curve', 'Cannot half-twist about this, it is not an essential curve bounding a pair of pants with a punctured complement.')
		else:
			tkMessageBox.showwarning('Incomplete triangulation', 'Cannot compute half twist when triangulation is incomplete.')
	
	def store_isometry(self):
		if self.is_complete():
			isometries = self.equipped_triangulation.triangulation.self_isometries()
			
			# Chop off the 'Isometry [' and ditch the ']'. Return at most max_char characters.
			max_char = 40
			name_shrinker = lambda strn: strn[10:].replace(']', '') if len(strn) < max_char + 11 else strn[10:7 + max_char] + '...'
			
			specification = flipper.application.get_choice('Available Isometries.', 'Use isometry mapping edges 0, 1, ... to: ',
				[name_shrinker(str(isom)) for isom in sorted(isometries, key=lambda isom: (isom.label_map[0] < 0, abs(isom.label_map[0])))])
			if specification is not None:
				[isometry] = [isom for isom in isometries if name_shrinker(str(isom)) == specification]
				name = flipper.application.get_input('Name', 'New isometry name:', validate=self.valid_name)
				if name is not None:
					self.add_mapping_class(isometry.encode(), name)
		else:
			tkMessageBox.showwarning('Incomplete triangulation', 'Cannot compute isometry when triangulation is incomplete.')
	
	def store_composition(self):
		if self.is_complete():
			composition = flipper.application.get_input('Composition', 'New composition:', validate=self.valid_composition)
			if composition is not None:
				name = flipper.application.get_input('Name', 'New composition name:', default=composition.replace('.', '_'), validate=self.valid_name)
				if name is not None:
					# self.valid_composition made sure that this wont fail.
					mapping_class = self.equipped_triangulation.mapping_class(composition)
					self.add_mapping_class(mapping_class, name)
		else:
			tkMessageBox.showwarning('Incomplete triangulation', 'Cannot compute composition when triangulation is incomplete.')
	
	
	######################################################################
	
	
	def canvas_left_click(self, event):
		self.canvas.focus_set()
		shift_pressed = (event.state & BIT_SHIFT) == BIT_SHIFT
		
		x, y = int(self.canvas.canvasx(event.x)), int(self.canvas.canvasy(event.y))
		possible_object = self.object_here((x, y))
		
		if self.is_complete() and not shift_pressed:
			if self.selected_object is None:
				if possible_object is None:
					self.select_object(self.create_curve_component([(x, y), (x, y)]))
			elif isinstance(self.selected_object, flipper.application.CurveComponent):
				if possible_object is not None:
					self.selected_object.append_point((x, y))
					self.canvas_to_lamination()
				else:
					self.canvas_to_lamination()
					self.select_object(None)
		else:
			if self.selected_object is None:
				if possible_object is None:
					self.select_object(self.create_vertex((x, y)))
				elif isinstance(possible_object, flipper.application.CanvasEdge):
					self.destroy_edge_identification(possible_object)
					if possible_object.free_sides() > 0:
						self.select_object(possible_object)
				elif isinstance(possible_object, flipper.application.CanvasVertex):
					self.select_object(possible_object)
				elif isinstance(possible_object, flipper.application.CurveComponent):
					self.select_object(possible_object)
			elif isinstance(self.selected_object, flipper.application.CanvasVertex):
				if possible_object == self.selected_object:
					self.select_object(None)
				elif possible_object is None:
					new_vertex = self.create_vertex((x, y))
					self.create_edge(self.selected_object, new_vertex)
					self.select_object(new_vertex)
				elif isinstance(possible_object, flipper.application.CanvasVertex):
					self.create_edge(self.selected_object, possible_object)
					self.select_object(possible_object)
				elif isinstance(possible_object, flipper.application.CanvasEdge):
					if possible_object.free_sides() > 0:
						self.select_object(possible_object)
			elif isinstance(self.selected_object, flipper.application.CanvasEdge):
				if possible_object == self.selected_object:
					self.select_object(None)
				elif possible_object is None:
					new_vertex = self.create_vertex((x, y))
					self.create_edge(self.selected_object[0], new_vertex)
					self.create_edge(self.selected_object[1], new_vertex)
					self.select_object(None)
				elif isinstance(possible_object, flipper.application.CanvasVertex):
					if possible_object != self.selected_object[0] and possible_object != self.selected_object[1]:
						self.create_edge(self.selected_object[0], possible_object)
						self.create_edge(self.selected_object[1], possible_object)
						self.select_object(None)
					else:
						self.select_object(possible_object)
				elif isinstance(possible_object, flipper.application.CanvasEdge):
					if (self.selected_object.free_sides() == 1 or self.selected_object.equivalent_edge is not None) and (possible_object.free_sides() == 1 or possible_object.equivalent_edge is not None):
						self.destroy_edge_identification(self.selected_object)
						self.destroy_edge_identification(possible_object)
						self.create_edge_identification(self.selected_object, possible_object)
						self.select_object(None)
					else:
						self.select_object(possible_object)
	
	def canvas_right_click(self, event):
		if self.selected_object is not None:
			if isinstance(self.selected_object, flipper.application.CurveComponent):
				if len(self.selected_object.vertices) > 2:
					(x, y) = self.selected_object.vertices[-1]
					self.selected_object.pop_point()
					self.selected_object.move_point(-1, x, y)
				else:
					self.destory_curve_component(self.selected_object)
					self.select_object(None)
				self.canvas_to_lamination()
			else:
				self.select_object(None)
	
	def canvas_move(self, event):
		x, y = int(self.canvas.canvasx(event.x)), int(self.canvas.canvasy(event.y))
		if isinstance(self.selected_object, flipper.application.CurveComponent):
			self.selected_object.move_point(-1, x, y)
	
	def canvas_focus_lost(self, event):
		self.select_object(None)
	
	def parent_key_press(self, event):
		key = event.keysym
		focus = self.parent.focus_get()
		if key in ('Delete', 'BackSpace'):
			if focus == self.canvas:
				if isinstance(self.selected_object, flipper.application.CanvasVertex):
					self.destroy_vertex(self.selected_object)
					self.select_object(None)
				elif isinstance(self.selected_object, flipper.application.CanvasEdge):
					self.destroy_edge(self.selected_object)
					self.select_object(None)
				elif isinstance(self.selected_object, flipper.application.CurveComponent):
					self.canvas_right_click(event)
			elif focus == self.treeview_objects:
				for iid in self.treeview_objects.selection():
					tags = self.treeview_objects.item(iid, 'tags')
					if 'lamination' in tags:
						self.remove_lamination(self.lamination_names[iid])
					elif 'mapping_class' in tags:
						self.remove_mapping_class(self.mapping_class_names[iid])
		elif key == 'Escape':
			self.canvas_right_click(event)
		elif key == 'F1':
			self.show_help()
		elif key == 'equal' or key == 'plus':
			self.zoom_in()
		elif key == 'minus' or key == 'underscore':
			self.zoom_centre(0.95)
		elif key == '0':
			self.zoom_to_drawing()
		elif key == 'Up':
			if focus == self.canvas:
				self.translate(0, 5)
		elif key == 'Down':
			if focus == self.canvas:
				self.translate(0, -5)
		elif key == 'Left':
			if focus == self.canvas:
				self.translate(5, 0)
		elif key == 'Right':
			if focus == self.canvas:
				self.translate(-5, 0)
	
	def treeview_objects_left_click(self, event):
		iid = self.treeview_objects.identify('row', event.x, event.y)
		tags = self.treeview_objects.item(iid, 'tags')
		if iid in self.lamination_names:
			name = self.lamination_names[iid]
		elif iid in self.mapping_class_names:
			name = self.mapping_class_names[iid]
		else:
			name = None
		
		if 'show_lamination' in tags:
			self.lamination_to_canvas(self.equipped_triangulation.laminations[name])
		elif 'apply_mapping_class' in tags:
			self.lamination_to_canvas(self.equipped_triangulation.mapping_classes[name](self.current_lamination))
		elif 'apply_mapping_class_inverse' in tags:
			self.lamination_to_canvas(self.equipped_triangulation.mapping_classes[name.swapcase()](self.current_lamination))
	
	def treeview_objects_double_left_click(self, event):
		self.treeview_objects_left_click(event)
		iid = self.treeview_objects.identify('row', event.x, event.y)
		tags = self.treeview_objects.item(iid, 'tags')
		if iid in self.lamination_names:
			name = self.lamination_names[iid]
		elif iid in self.mapping_class_names:
			name = self.mapping_class_names[iid]
		else:
			name = None
		
		if 'mapping_class_type' in tags:
			try:
				result = self.update_cache(self.equipped_triangulation.mapping_classes[name].nielsen_thurston_type)
				
				self.treeview_objects.item(iid, text='Type: %s' % result)
				self.unsaved_work = True
			except flipper.AbortError:
				pass
		elif 'mapping_class_invariant_lamination' in tags:
			try:
				result = self.update_cache(self.equipped_triangulation.mapping_classes[name].invariant_lamination)
				
				self.lamination_to_canvas(result)
				self.unsaved_work = True
			except flipper.AssumptionError:
				tkMessageBox.showwarning('Lamination', 'Cannot find any projectively invariant laminations, mapping class is not pseudo-Anosov.')
				self.unsaved_work = True
			except flipper.AbortError:
				pass
		elif 'mapping_class_conjugate' in tags:
			other = flipper.application.get_choice('Conjugate.', 'Is this mapping class conjugate to...',
				sorted(set(self.mapping_class_names.values()), key=lambda x: (len(x), x)))
			if other is not None:
				try:
					result = self.update_cache(
						self.equipped_triangulation.mapping_classes[name].is_conjugate_to,
						args=(self.equipped_triangulation.mapping_classes[other],)
						)
					
					if result:
						tkMessageBox.showinfo('Conjugate', '%s and %s are conjugate.' % (name, other))
					else:
						tkMessageBox.showinfo('Conjugate', '%s and %s are not conjugate.' % (name, other))
					
					self.unsaved_work = True
				except flipper.AssumptionError:
					tkMessageBox.showwarning('Conjugate', 'Could not determine conjugacy, mapping class is not pseudo-Anosov.')
					self.unsaved_work = True
				except flipper.AbortError:
					pass
		elif 'mapping_class_bundle' in tags:
			path = tkFileDialog.asksaveasfilename(defaultextension='.tri', filetypes=[('SnapPy Files', '.tri'), ('all files', '.*')], title='Export SnapPy Triangulation')
			if path != '':
				try:
					bundle = self.update_cache(self.equipped_triangulation.mapping_classes[name].bundle)
					with open(path, 'wb') as disk_file:
						disk_file.write(bundle.snappy_string())
				except flipper.AssumptionError:
					tkMessageBox.showwarning('Bundle', 'Cannot build bundle, mapping class is not pseudo-Anosov.')
				except flipper.AbortError:
					pass
				except IOError:
					tkMessageBox.showwarning('Save Error', 'Could not write to: %s' % path)
		else:
			pass
	
	def update_cache(self, method, args=None):
		# So we need to be really careful here. Linux uses copy-on-write (COW) so
		# when we spawn a new thread in a progress bar if we ever do something
		# non-pure and write into that data structure a copy is taken first.
		# This means that things may not match up later as, for example, laminations
		# now exist on a copy of this triangulation.
		#
		# I guess this is a strong argument in favour of making functions / methods pure.
		
		if args is None: args = []
		self.equipped_triangulation, result = flipper.application.apply_progression(helper, args=(self.equipped_triangulation, method, args))
		
		# Don't forget that what is drawn on the canvas is still thought of as a lamination on the old triangulation.
		self.canvas_to_lamination()
		
		return result


def start(load_from=None):
	root = TK.Tk()
	root.title('flipper')
	flipper_application = FlipperApplication(root)
	root.minsize(300, 300)
	root.geometry('700x500')
	root.wait_visibility(root)
	if load_from is not None: flipper_application.load(load_from=load_from)
	# Set the icon.
	# Make sure to get the right path if we are in a cx_Freeze compiled executable.
	# See: http://cx-freeze.readthedocs.org/en/latest/faq.html#using-data-files
	datadir = os.path.dirname(sys.executable if getattr(sys, 'frozen', False) else __file__)
	icon_path = os.path.join(datadir, 'icon', 'icon.gif')
	img = TK.PhotoImage(file=icon_path)
	try:
		root.tk.call('wm', 'iconphoto', root._w, img)
	except TK.TclError:
		# Give up if we can't set the icon for some reason.
		# This seems to be a problem if you start flipper within SnapPy.
		pass
	root.mainloop()
	return flipper_application.output

if __name__ == '__main__':
	start()





from __future__ import print_function
from time import time
import cProfile
import pstats

import flipper

def main(verbose=False):
	if verbose: print('Running encoding profile.')
	
	start_time = time()
	# Add more tests here.
	# These need to be changed if the standard example triangulations ever change.
	
	for i in range(10, 30, 3):
		if verbose: print('Building B_%d' % i)
		S = flipper.load('B_%d' % i)
	
	return time() - start_time

if __name__ == '__main__':
	#print(main(verbose=True))
	#pstats.Stats(cProfile.Profile().run('main()')).strip_dirs().sort_stats('time').print_callers(20)
	#pstats.Stats(cProfile.Profile().run('main()')).strip_dirs().sort_stats('cumtime').print_callees(20)
	pstats.Stats(cProfile.Profile().run('main()')).strip_dirs().sort_stats('time').print_stats(30)





from __future__ import print_function
from time import time

import flipper

def main():
	for surface, word, target in flipper.census('knot_monodromies'):
		print('Buiding: %s over %s (target %s).' % (word, surface, target))
		start_time = time()
		stratum = flipper.load(surface).mapping_class(word).stratum()
		vertex_orders = [stratum[singularity] for singularity in stratum]
		# A singularity is real if and only if its label is non-negative.
		real_vertex_orders = [stratum[singularity] for singularity in stratum if not singularity.filled]
		# This should be snappy.twister.Surface(surface).bundle(word) == snappy.Manifold(target)
		# But we wont bother to check this here.
		print('\tAll: %s, Real: %s' % (vertex_orders, real_vertex_orders))

if __name__ == '__main__':
	import argparse
	parser = argparse.ArgumentParser(description='Compute the singularity orders of knot complements.')
	parser.add_argument('--show', action='store_true', default=False, help='show the source code of this example and exit')
	args = parser.parse_args()
	
	if args.show:
		print(open(__file__, 'r').read())
	else:
		main()





from __future__ import print_function
try:
	import snappy
except ImportError:
	snappy = None

import flipper

def test(surface, word, target):
	snappy_string = flipper.load(surface).mapping_class(word).bundle(canonical=False).snappy_string()
	# Snappy can fail with a RuntimeError, particularly when canonical=False.
	M = snappy.Manifold(snappy_string)
	for _ in range(100):
		try:
			if M.is_isometric_to(target):
				return True
		except RuntimeError:
			pass  # SnapPy couldn't decide if these are isometric or not.
		M.randomize()
	
	return False

def main(verbose=False):
	if verbose: print('Running layered triangulation tests.')
	
	if snappy is None:  # We really should check that the version is > 2.0.4 so that twister exists.
		print('SnapPy required but unavailable, tests skipped.')
		return True
	
	tests = [
		('S_1_1', 'aB', 'm004'),
		('S_1_1', 'Ba', 'm004'),
		('S_1_1', 'Ab', 'm004'),
		('S_1_1', 'bA', 'm004'),
		('S_1_1', 'aBababab', 'm003'),
		('S_1_1', 'Baababab', 'm003'),
		('S_1_1', 'Abababab', 'm003'),
		('S_1_1', 'bAababab', 'm003'),
		('S_1_1', 'aBaB', 'm206'), ('S_1_1', 'aBaBababab', 'm207'),  # Double covers.
		('S_1_2', 'abC', 'm129'),
		('S_2_1', 'aaabcd', 'm036'),
		('S_2_1', 'abcdeF', 'm038')
		]
	
	twister_tests = [
		('S_1_1', 'aB')
		]
	
	for surface, word, target_manifold in tests:
		if verbose: print(word)
		if not test(surface, word, snappy.Manifold(target_manifold)):
			return False
	for surface, word in twister_tests:
		if verbose: print(word)
		if not test(surface, word, snappy.twister.Surface(surface).bundle(word)):
			return False
	
	return True

if __name__ == '__main__':
	print(main(verbose=True))





import flipper

import os
import doctest
from importlib import import_module

def run_tests(verbose=False):
	failed_tests = []
	num_failed, num_attempted = 0, 0
	for filename in dir(flipper.kernel):
		try:
			mod = import_module('flipper.kernel.' + filename)
		except ImportError:
			pass
		else:
			if verbose: print('Running %s test...' % filename)
			results = doctest.testmod(mod, extraglobs=mod.doctest_globs() if hasattr(mod, 'doctest_globs') else {})
			if results.failed == 0:
				if verbose: print('\t%s' % str(results))
			else:
				failed_tests.append(filename)
			num_failed += results.failed
			num_attempted += results.attempted
	
	if verbose:
		print('Attempted: %d' % num_attempted)
		print('Failed: %d' % num_failed)
	if failed_tests:
		print('FAILED TESTS:')
		for test in failed_tests:
			print('\t%s' % test)
		return False
	else:
		print('All tests passed.')
		return True

if __name__ == '__main__':
	run_tests(verbose=True)





from __future__ import print_function
from time import time
import cProfile
import pstats

import flipper

NT_TYPE_PERIODIC = flipper.kernel.encoding.NT_TYPE_PERIODIC
NT_TYPE_REDUCIBLE = flipper.kernel.encoding.NT_TYPE_REDUCIBLE
NT_TYPE_PSEUDO_ANOSOV = flipper.kernel.encoding.NT_TYPE_PSEUDO_ANOSOV

def main(verbose=False):
	if verbose: print('Running encoding profile.')
	
	start_time = time()
	# Add more tests here.
	tests = [
		('S_1_1', 'a', NT_TYPE_REDUCIBLE),
		('S_1_2', 'a', NT_TYPE_REDUCIBLE),
		('S_1_2', 'b', NT_TYPE_REDUCIBLE),
		('S_1_2', 'c', NT_TYPE_REDUCIBLE),
		('S_1_2', 'aB', NT_TYPE_REDUCIBLE),
		('S_1_2', 'bbaCBAaBabcABB', NT_TYPE_REDUCIBLE),
		('S_1_2', 'aCBACBacbaccbAaAcAaBBcCcBBcCaBaaaABBabBcaBbCBCbaaa', NT_TYPE_PSEUDO_ANOSOV),
		('S_2_1', 'aaabcd', NT_TYPE_PSEUDO_ANOSOV),
		('E_12', 'aaaaBBp', NT_TYPE_PSEUDO_ANOSOV),
		('E_12', 'aaBaaBBp', NT_TYPE_REDUCIBLE),
		('E_12', 'aaaaBBaBaBp', NT_TYPE_PSEUDO_ANOSOV),
		]
	
	for surface, word, _ in tests:
		if verbose: print(word)
		mapping_class = flipper.load(surface).mapping_class(word)
		try:
			mapping_class.invariant_lamination()
		except flipper.AssumptionError:
			# The mapping class was not pseudo-Anosov.
			pass
	
	return time() - start_time

if __name__ == '__main__':
	# main()
	# print(main(verbose=True))
	#pstats.Stats(cProfile.Profile().run('main()')).strip_dirs().sort_stats('time').print_callers(20)
	#pstats.Stats(cProfile.Profile().run('main()')).strip_dirs().sort_stats('cumtime').print_callees(20)
	pstats.Stats(cProfile.Profile().run('main()')).strip_dirs().sort_stats('time').print_stats(30)





from __future__ import print_function

import flipper

algebraic_approximation = flipper.kernel.AlgebraicApproximation.from_tuple

def main(verbose=False):
	if verbose: print('Running algebraic approximation tests.')
	
	# We acutally need quite a lot of accuracy in the given strings.
	x = algebraic_approximation('1.4142135623730950488016887242096980785696718753769480', 2, 2)
	y = algebraic_approximation('1.4142135623730950488016887242096980', 2, 2)
	z = algebraic_approximation('1.000000000000', 2, 2)
	
	tests = [
		x == y,
		z != y,
		x + y == x + x,
		x * x == 2,
		y * y == 2,
		y * y + x == 2 + x,
		y * (y + y) == 4,
		x * x == y * y,
		x + x > 0,
		-(x + x) < 0,
		x > z > 0
		]
	
	return all(tests)

if __name__ == '__main__':
	print(main(verbose=True))





from __future__ import print_function
from itertools import product, permutations

import flipper

def main(verbose=False, n=4):
	if verbose: print('Running permutation tests.')
	
	# Get some example permutations.
	all_perms = flipper.kernel.permutation.all_permutations(n)
	
	# Check that there are the same number of odd and even permutations.
	if not (len([perm for perm in all_perms if perm.is_even()]) == len([perm for perm in all_perms if not perm.is_even()])): return False
	# Check that composition respects parity.
	if not (all((p1 * p2).is_even() == p1.is_even() ^ p2.is_even() ^ True for p1, p2 in product(all_perms, all_perms))): return False
	# Check that every permutation acts transitively.
	if not (all(set(perm * p1 for p1 in all_perms) == set(all_perms) for perm in all_perms)): return False
	
	return True

if __name__ == '__main__':
	print(main(verbose=True))





import flipper
import flipper.application
from flipper.application.spinning_icon import SPINNING_ICON

from multiprocessing import Process
from multiprocessing import JoinableQueue as Queue

try:
	from Queue import Empty
except ImportError:  # Python 3.
	from queue import Empty

try:
	import Tkinter as TK
except ImportError:  # Python 3.
	import tkinter as TK

# Note: Because of how memory is managed in Python the _worker_thread
# works with a copy of the main memory. Therefore any changes it makes to the
# objects involved, such as caching results, will NOT be done to the origional
# objects. Hence all such changes will be lost when the thread completes.
def _worker_thread(function, args, answer):
	try:
		result = function(*args)
		answer.put(result)
	except Exception as error:
		answer.put(error)  # Return any errors that occur.

class ProgressApp(object):
	def __init__(self, host_app_parent=None):
		if host_app_parent is None: host_app_parent = TK._default_root
		self.host_app_parent = host_app_parent
		self.parent = TK.Toplevel(self.host_app_parent)
		self.parent.withdraw()  # Hide self while we set up the geometry.
		
		self.parent.title('flipper: Computing...')
		self.parent.protocol('WM_DELETE_WINDOW', self.cancel)  # To catch when users click the 'x' to close the window.
		
		self.progress = flipper.application.AnimatedCanvas(self.parent, frames_contents=SPINNING_ICON)
		self.progress.pack(padx=130, pady=5)
		
		self.button_cancel = TK.Button(self.parent, text='Cancel', command=self.cancel)
		self.button_cancel.pack(padx=5, pady=5, side='right')
		
		self.running = False
		self.worker = None
		
		self.parent.resizable(0, 0)
		self.parent.withdraw()
		self.parent.lift()
		self.button_cancel.focus()
		self.parent.update_idletasks()
		x = self.host_app_parent.winfo_rootx() + self.host_app_parent.winfo_width() // 2 - self.parent.winfo_width() // 2
		y = self.host_app_parent.winfo_rooty() + self.host_app_parent.winfo_height() // 2 - self.parent.winfo_height() // 2
		self.parent.update_idletasks()
		self.parent.geometry('+%d+%d' % (x, y))
		self.parent.deiconify()
		self.parent.transient(self.host_app_parent)  # Lock this window on top.
		self.parent.grab_set()  # Make sure this window always has focus.
		self.parent.update_idletasks()
	
	def cancel(self):
		if self.running:
			self.running = False
			self.worker.terminate()
			self.host_app_parent.focus_set()
			self.parent.destroy()
	
	def apply(self, function, args=None):
		if args is None: args = []
		answer = Queue()
		self.worker = Process(target=_worker_thread, args=(function, args, answer))
		self.worker.deamon = True  # Making the worker a deamon of this thread stops it if the main thread is killed.
		self.running = True
		self.worker.start()
		
		while self.running:  # So long as the calculation hasn't been aborted.
			try:
				result = answer.get(True, 0.05)  # Try and get some more information.
			except Empty:
				self.host_app_parent.update()
			else:
				self.cancel()
				if isinstance(result, Exception):
					raise result
				else:
					return result
		
		# If we reach this point then the calculation was aborted.
		raise flipper.AbortError

def apply_progression(function, args=None, host_app_parent=None):
	return ProgressApp(host_app_parent).apply(function, args)

